@@ -257,6 +257,7 @@ struct Odt_parameters
   double time_limit;
   double convergence_ratio;
   double freeze_ratio;
+  bool do_freeze;
   int max_iteration_nb;
   
   QStringList log() const
@@ -265,6 +266,7 @@ struct Odt_parameters
       << QString("time limit: %1").arg(time_limit)
       << QString("convergence ratio: %1").arg(convergence_ratio)
       << QString("freeze ratio: %1").arg(freeze_ratio)
+      << QString("do freeze: %1").arg(do_freeze)
       << QString("maximum iterations: %1").arg(max_iteration_nb);
   }
 };
@@ -281,8 +283,6 @@ class Optimization_function < Domain, Odt_parameters >
 {
   // Private types
   typedef C3t3::Triangulation  Tr;
-  typedef Tr::Geom_traits      Gt;
-  
   typedef CGAL::Mesh_3::Mesh_sizing_field<Tr>    Sizing;
   typedef CGAL::Mesh_3::Odt_move<C3t3,Sizing>    Move;
   typedef Global_visitor                         Visitor;
@@ -317,7 +317,7 @@ protected:
     if ( NULL != odt_ ) { return CGAL::MESH_OPTIMIZATION_UNKNOWN_ERROR; }
 
     // Create optimizer
-    odt_ = new Odt_optimizer(c3t3, domain, p_.freeze_ratio, p_.convergence_ratio);
+    odt_ = new Odt_optimizer(c3t3, domain, p_.freeze_ratio, p_.do_freeze, p_.convergence_ratio);
     if ( NULL == odt_ ) { return CGAL::MESH_OPTIMIZATION_UNKNOWN_ERROR; }
     
     // Set max time
@@ -367,6 +367,7 @@ struct Lloyd_parameters
   double time_limit;
   double convergence_ratio;
   double freeze_ratio;
+  bool do_freeze;
   int max_iteration_nb;
   
   QStringList log() const
@@ -375,6 +376,7 @@ struct Lloyd_parameters
       << QString("time limit: %1").arg(time_limit)
       << QString("convergence ratio: %1").arg(convergence_ratio)
       << QString("freeze ratio: %1").arg(freeze_ratio)
+      << QString("do freeze: %1").arg(do_freeze)
       << QString("maximum iterations: %1").arg(max_iteration_nb);
   }
 };
@@ -391,8 +393,6 @@ class Optimization_function < Domain, Lloyd_parameters >
 {
   // Private types
   typedef C3t3::Triangulation  Tr;
-  typedef Tr::Geom_traits      Gt;
-  
   typedef CGAL::Mesh_3::Mesh_sizing_field<Tr>    Sizing;
   typedef CGAL::Mesh_3::Lloyd_move<C3t3,Sizing>  Move;
   typedef Global_visitor                         Visitor;
@@ -427,7 +427,7 @@ protected:
     if ( NULL != lloyd_ ) { return CGAL::MESH_OPTIMIZATION_UNKNOWN_ERROR; }
     
     // Create optimizer
-    lloyd_ = new Lloyd_optimizer(c3t3, domain, p_.freeze_ratio, p_.convergence_ratio);
+    lloyd_ = new Lloyd_optimizer(c3t3, domain, p_.freeze_ratio, p_.do_freeze, p_.convergence_ratio);
     if ( NULL == lloyd_ ) { return CGAL::MESH_OPTIMIZATION_UNKNOWN_ERROR; }
     
     // Set max time
@@ -517,8 +517,8 @@ class Optimization_function < Domain, Perturb_parameters >
   : public Optimization_function_base< Domain >
 {
   // Private types
-  typedef C3t3::Triangulation::Geom_traits                        Gt;
-  typedef CGAL::Mesh_3::Min_dihedral_angle_criterion<Gt>          Sc;
+  typedef C3t3::Triangulation                                     Tr;
+  typedef CGAL::Mesh_3::Min_dihedral_angle_criterion<Tr>          Sc;
   typedef Perturb_visitor                                         Visitor;
   
   typedef CGAL::Mesh_3::Sliver_perturber<C3t3,Domain,Sc,Visitor>  Perturber;
@@ -530,7 +530,8 @@ public:
   Optimization_function(C3t3& c3t3, Domain* d, const Perturb_parameters& p)
     : Base(c3t3,d)
     , perturb_(NULL)
-    , p_(p) {}
+    , p_(p)
+    , criterion_(p.sliver_bound, c3t3.triangulation()) {}
   
   /// Destructor
   ~Optimization_function() { delete perturb_; }
@@ -563,7 +564,7 @@ protected:
     typedef CGAL::Mesh_3::Li_random_perturbation<C3t3,Domain,Sc>      Li_random;
     
     // Build perturber
-    perturb_ = new Perturber(c3t3,domain);
+    perturb_ = new Perturber(c3t3, domain, criterion_);
     if ( NULL == perturb_ ) { return CGAL::MESH_OPTIMIZATION_UNKNOWN_ERROR; }
     
     // Add perturbations
@@ -579,13 +580,14 @@ protected:
     if ( 0 == p_.sliver_bound ) { p_.sliver_bound = Sc::max_value; }
     
     // Launch perturber
-    return (*perturb_)(p_.sliver_bound, 1, Visitor(&status_));
+    return (*perturb_)(Visitor(&status_));
   }
 
 private:
   Perturber* perturb_;
   Perturb_parameters p_;
   Perturb_status status_;
+  Sc criterion_;
 };
 
 
@@ -653,8 +655,8 @@ class Optimization_function < Domain, Exude_parameters >
   : public Optimization_function_base< Domain >
 {
   // Private types
-  typedef C3t3::Triangulation::Geom_traits                  Gt;
-  typedef CGAL::Mesh_3::Min_dihedral_angle_criterion<Gt>    Sc;
+  typedef C3t3::Triangulation                               Tr;
+  typedef CGAL::Mesh_3::Min_dihedral_angle_criterion<Tr>    Sc;
   typedef Exude_visitor                                     Visitor;
   typedef CGAL::Mesh_3::Slivers_exuder<C3t3,Sc,Visitor>     Exuder;
   
@@ -665,7 +667,8 @@ public:
   Optimization_function(C3t3& c3t3, Domain* d, const Exude_parameters& p)
     : Base(c3t3,d)
     , exude_(NULL)
-    , p_(p) {}
+    , p_(p)
+    , criterion_(p.sliver_bound, c3t3.triangulation()) {}
   
   /// Destructor
   ~Optimization_function() { delete exude_; }
@@ -693,20 +696,21 @@ protected:
     if ( NULL != exude_ ) { return CGAL::MESH_OPTIMIZATION_UNKNOWN_ERROR; }
     
     // Create exuder
-    exude_ = new Exuder(c3t3);
+    exude_ = new Exuder(c3t3,criterion_);
     if ( NULL == exude_ ) { return CGAL::MESH_OPTIMIZATION_UNKNOWN_ERROR; }
     
     // Set time_limit
     exude_->set_time_limit(p_.time_limit);
     
     // Launch exudation
-    return (*exude_)(p_.sliver_bound, Visitor(&status_));
+    return (*exude_)(Visitor(&status_));
   }
   
 private:
   Exuder* exude_;
   Exude_parameters p_;
   Exude_status status_;
+  Sc criterion_;
 };
 
 
