@@ -6,14 +6,16 @@
 #include <CGAL/AABB_tree.h>
 #include <CGAL/AABB_traits.h>
 #include <CGAL/Polyhedron_3.h>
-#include <CGAL/AABB_polyhedron_segment_primitive.h>
+#include <CGAL/AABB_halfedge_graph_segment_primitive.h>
+#include <CGAL/boost/graph/halfedge_graph_traits_Polyhedron_3.h>
+#include <CGAL/boost/graph/properties_Polyhedron_3.h>
 
 typedef CGAL::Simple_cartesian<double> K;
 typedef K::FT FT;
 typedef K::Point_3 Point;
 typedef K::Triangle_3 Triangle;
 typedef CGAL::Polyhedron_3<K> Polyhedron;
-typedef CGAL::AABB_polyhedron_segment_primitive<K,Polyhedron> Primitive;
+typedef CGAL::AABB_halfedge_graph_segment_primitive<Polyhedron> Primitive;
 typedef CGAL::AABB_traits<K, Primitive> Traits;
 typedef CGAL::AABB_tree<Traits> Tree;
 
@@ -28,7 +30,9 @@ int main()
 
     // constructs the AABB tree and the internal search tree for 
     // efficient distance queries.
-    Tree tree(polyhedron.edges_begin(),polyhedron.edges_end());
+    Tree tree( CGAL::undirected_edges(polyhedron).first,
+               CGAL::undirected_edges(polyhedron).second,
+               polyhedron);
     tree.accelerate_distance_queries();
 
     // counts #intersections with a triangle query
