@@ -15,7 +15,7 @@
 #include <CGAL/Mesh_complex_3_in_triangulation_3.h>
 #include <CGAL/Mesh_criteria_3.h>
 
-#include <CGAL/Mesh_3/implicit_to_labeled_function_wrapper.h>
+#include <CGAL/Mesh_3/Implicit_to_labeled_function_wrapper.h>
 #include <CGAL/Mesh_3/Labeled_mesh_domain_3.h>
 #include <CGAL/make_mesh_3.h>
 #include "implicit_functions.h"
