@@ -12,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/releases/CGAL-4.0-branch/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_polygon_validation.h $
-// $Id: Gps_polygon_validation.h 67117 2012-01-13 18:14:48Z lrineau $
+// $URL$
+// $Id$
 // 
 //
 // Author(s): Baruch Zukerman <baruchzu@post.tau.ac.il>
@@ -41,17 +41,6 @@
 #include <list>
 #include <iterator>
 
-#define CGAL_GPS_POLYGON_VALIDATION_2_TYPEDEF                           \
-  typedef Gps_traits_adaptor<Traits_2>              Traits_adapter_2;   \
-  typedef typename Traits_2::Curve_const_iterator                       \
-    Curve_const_iterator;                                               \
-  typedef std::pair<Curve_const_iterator,Curve_const_iterator>          \
-    Cci_pair;                                                           \
-  typedef typename Traits_2::Construct_curves_2                         \
-    Construct_curves_2;                                                 \
-  typedef typename Traits_adapter_2::Construct_vertex_2                 \
-    Construct_vertex_2
-  
 
 namespace CGAL {
 
@@ -79,7 +68,7 @@ public:
    * caused by the PWH's outer boundary
    */
   virtual void create_face(Face_handle_A red_face, Face_handle_B blue_face,
-                           Face_handle_R /*r_face*/) {    
+                           Face_handle_R /*r_face*/) const {    
     if ((red_face->contained()==true) && (blue_face->contained()==false)) {
       hole_overlap = true;
     }
@@ -95,7 +84,7 @@ public:
     return;
   }
 private:    
-  bool hole_overlap;
+  mutable bool hole_overlap;
 }; 
   
 /*! \class
@@ -187,8 +176,12 @@ private:
 template <typename Traits_2>
 bool is_closed_polygon(const typename Traits_2::Polygon_2& pgn, Traits_2 traits)
 {
+
+  typedef Gps_traits_adaptor<Traits_2>              Traits_adapter_2;
+  typedef typename Traits_2::Curve_const_iterator    Curve_const_iterator;
+  typedef std::pair<Curve_const_iterator,Curve_const_iterator>    Cci_pair; 
+  typedef typename Traits_adapter_2::Construct_vertex_2 Construct_vertex_2;
   
-  CGAL_GPS_POLYGON_VALIDATION_2_TYPEDEF;
   Cci_pair              itr_pair = traits.construct_curves_2_object()(pgn);
   Curve_const_iterator  begin = itr_pair.first;
   Curve_const_iterator  end = itr_pair.second;
@@ -238,8 +231,9 @@ bool is_closed_polygon(const typename Traits_2::Polygon_2& pgn, Traits_2 traits)
 template <typename Traits_2>
 bool is_simple_polygon(const typename Traits_2::Polygon_2& pgn, Traits_2 traits)
 {
+  typedef typename Traits_2::Curve_const_iterator    Curve_const_iterator;
+  typedef std::pair<Curve_const_iterator,Curve_const_iterator>    Cci_pair; 
 
-  CGAL_GPS_POLYGON_VALIDATION_2_TYPEDEF;
   // Sweep the boundary curves and look for intersections.
   typedef Gps_polygon_validation_visitor<Traits_2>  Visitor;
   typedef Sweep_line_2<Traits_2, Visitor>           Sweep_line;
@@ -277,11 +271,13 @@ template <typename Traits_2>
 bool has_valid_orientation_polygon (const typename Traits_2::Polygon_2& pgn,
                                     Traits_2 traits)
 {
-  CGAL_GPS_POLYGON_VALIDATION_2_TYPEDEF;
+
+  typedef Gps_traits_adaptor<Traits_2>              Traits_adapter_2;
+  typedef typename Traits_2::Curve_const_iterator    Curve_const_iterator;
+  typedef std::pair<Curve_const_iterator,Curve_const_iterator>    Cci_pair; 
 
   Cci_pair         itr_pair = traits.construct_curves_2_object()(pgn);
   Traits_adapter_2 traits_adapter;
-  typedef typename Traits_adapter_2::Orientation_2  Check_orientation_2;
 
   if(itr_pair.first == itr_pair.second)
     return true; // empty polygon
@@ -344,7 +340,9 @@ bool
 is_crossover_outer_boundary(const typename Traits_2::Polygon_with_holes_2& pgn,
                             Traits_2 traits, PointLocation& pl)
 {
-  CGAL_GPS_POLYGON_VALIDATION_2_TYPEDEF;
+  typedef typename Traits_2::Curve_const_iterator    Curve_const_iterator;
+  typedef std::pair<Curve_const_iterator,Curve_const_iterator>    Cci_pair; 
+
   typedef typename Traits_2::Point_2                 Point_2;
   typedef typename Traits_2::Compare_endpoints_xy_2  Compare_endpoints_xy_2;
   typedef typename Traits_2::Construct_min_vertex_2  Construct_min_vertex_2; 
@@ -358,8 +356,7 @@ is_crossover_outer_boundary(const typename Traits_2::Polygon_with_holes_2& pgn,
                                                      Topology_traits;
   typedef CGAL::Gps_on_surface_base_2<Traits_2, Topology_traits> 
     Polygon_set_2;
-  typedef typename Traits_2::Polygon_with_holes_2
-    Polygon_with_holes_2;
+
   typedef typename Polygon_set_2::Arrangement_on_surface_2 Arrangement_2;
   typedef typename Arrangement_2::Halfedge_handle          Halfedge_handle;
   typedef typename Arrangement_2::Vertex_handle            Vertex_handle;
@@ -488,7 +485,10 @@ template <typename Traits_2>
 bool is_relatively_simple_polygon_with_holes
 (const typename Traits_2::Polygon_with_holes_2& pgn, Traits_2 traits)
 {
-  CGAL_GPS_POLYGON_VALIDATION_2_TYPEDEF;
+  typedef typename Traits_2::Curve_const_iterator    Curve_const_iterator;
+  typedef std::pair<Curve_const_iterator,Curve_const_iterator>    Cci_pair; 
+  typedef typename Traits_2::Construct_curves_2    Construct_curves_2; 
+
   typedef typename Traits_2::X_monotone_curve_2     X_monotone_curve_2;
   typedef Gps_polygon_validation_visitor<Traits_2>  Visitor;
   typedef Sweep_line_2<Traits_2, Visitor>           Sweep_line;
@@ -542,10 +542,12 @@ template <typename Traits_2>
 bool has_valid_orientation_polygon_with_holes
 (const typename Traits_2::Polygon_with_holes_2& pgn, Traits_2 traits)
 {
-  CGAL_GPS_POLYGON_VALIDATION_2_TYPEDEF;
-  typedef typename Traits_2::X_monotone_curve_2       X_monotone_curve_2;
-  typedef Gps_polygon_validation_visitor<Traits_2>      Visitor;
-  typedef Sweep_line_2<Traits_2, Visitor>               Sweep_line;
+
+  typedef Gps_traits_adaptor<Traits_2>              Traits_adapter_2;
+  typedef typename Traits_2::Curve_const_iterator    Curve_const_iterator;
+  typedef std::pair<Curve_const_iterator,Curve_const_iterator>    Cci_pair; 
+  typedef typename Traits_2::Construct_curves_2    Construct_curves_2; 
+
   typedef typename Traits_adapter_2::Orientation_2      Check_orientation_2;
   typedef typename Traits_2::Polygon_with_holes_2       Polygon_with_holes_2;
 
@@ -597,7 +599,9 @@ template <class Traits_2>
 bool are_holes_and_boundary_pairwise_disjoint
 (const typename Traits_2::Polygon_with_holes_2& pwh, Traits_2& traits)
 {
-  CGAL_GPS_POLYGON_VALIDATION_2_TYPEDEF;
+  typedef typename Traits_2::Curve_const_iterator    Curve_const_iterator;
+  typedef std::pair<Curve_const_iterator,Curve_const_iterator>    Cci_pair; 
+  typedef typename Traits_2::Construct_curves_2    Construct_curves_2; 
 
   typedef CGAL::Gps_default_dcel<Traits_2>                 Dcel;
   // IMPORTATNT! TODO!
@@ -619,7 +623,7 @@ bool are_holes_and_boundary_pairwise_disjoint
   typedef typename Traits_2::Construct_curves_2            Construct_curves_2;
   typedef typename Traits_2::Construct_general_polygon_with_holes_2
     Construct_polygon_with_holes_2;
-  typedef typename Traits_adapter_2::Construct_vertex_2    Construct_vertex_2;
+
   typedef Gps_polygon_validation_visitor<Traits_2>         Visitor;
   typedef Sweep_line_2<Traits_2, Visitor>                  Sweep_line ;
   typedef typename Polygon_set_2::Arrangement_on_surface_2 Arrangement_2;
