@@ -24,6 +24,35 @@
 #include <CGAL/triangulation_assertions.h>
 #include <CGAL/Constrained_triangulation_2.h>
 
+#ifndef CGAL_TRIANGULATION_2_DONT_INSERT_RANGE_OF_POINTS_WITH_INFO
+#include <CGAL/Spatial_sort_traits_adapter_2.h>
+#include <CGAL/internal/info_check.h>
+#include <CGAL/is_iterator.h>
+
+#include <boost/iterator/zip_iterator.hpp>
+#include <boost/mpl/and.hpp>
+
+#include <boost/iterator/counting_iterator.hpp>
+
+namespace CGAL {
+
+namespace internal{
+
+template <class T,bool has_info=is_iterator<T>::value>
+struct Get_iterator_value_type{
+ struct type{};
+};
+
+template <class T>
+struct Get_iterator_value_type<T,true>{
+ typedef typename std::iterator_traits<T>::value_type type;
+};
+
+} } //namespace CGAL::internal
+#endif //CGAL_TRIANGULATION_2_DONT_INSERT_RANGE_OF_POINTS_WITH_INFO
+
+
+
 namespace CGAL {
 
 
@@ -196,10 +225,22 @@ public:
   //template member functions
 public:
   template < class InputIterator >
+#ifndef CGAL_TRIANGULATION_2_DONT_INSERT_RANGE_OF_POINTS_WITH_INFO
+  std::ptrdiff_t
+  insert( InputIterator first, InputIterator last,
+          typename boost::enable_if<
+            boost::is_convertible<
+                typename internal::Get_iterator_value_type< InputIterator >::type,
+                Point
+            >
+          >::type* = NULL
+  )
+#else
 #if defined(_MSC_VER)
   std::ptrdiff_t insert(InputIterator first, InputIterator last, int i = 0)
 #else
     std::ptrdiff_t insert(InputIterator first, InputIterator last) 
+#endif
 #endif
     {
       size_type n = number_of_vertices();
@@ -214,6 +255,177 @@ public:
       return number_of_vertices() - n;
     }
 
+#ifndef CGAL_TRIANGULATION_2_DONT_INSERT_RANGE_OF_POINTS_WITH_INFO
+private:
+  //top stands for tuple-or-pair
+  template <class Info>
+  const Point& top_get_first(const std::pair<Point,Info>& pair) const { return pair.first; }
+  template <class Info>
+  const Info& top_get_second(const std::pair<Point,Info>& pair) const { return pair.second; }
+  template <class Info>
+  const Point& top_get_first(const boost::tuple<Point,Info>& tuple) const { return boost::get<0>(tuple); }
+  template <class Info>
+  const Info& top_get_second(const boost::tuple<Point,Info>& tuple) const { return boost::get<1>(tuple); }
+
+  template <class Tuple_or_pair,class InputIterator>
+  std::ptrdiff_t insert_with_info(InputIterator first,InputIterator last)
+  {
+    size_type n = this->number_of_vertices();
+    std::vector<std::ptrdiff_t> indices;
+    std::vector<Point> points;
+    std::vector<typename Tds::Vertex::Info> infos;
+    std::ptrdiff_t index=0;
+    for (InputIterator it=first;it!=last;++it){
+      Tuple_or_pair value=*it;
+      points.push_back( top_get_first(value)  );
+      infos.push_back ( top_get_second(value) );
+      indices.push_back(index++);
+    }
+
+    typedef Spatial_sort_traits_adapter_2<Geom_traits,Point*> Search_traits;
+
+    spatial_sort(indices.begin(),indices.end(),Search_traits(&(points[0]),geom_traits()));
+
+    Vertex_handle v_hint;
+    Face_handle hint;
+    for (typename std::vector<std::ptrdiff_t>::const_iterator
+      it = indices.begin(), end = indices.end();
+      it != end; ++it){
+      v_hint = insert(points[*it], hint);
+      if (v_hint!=Vertex_handle()){
+        v_hint->info()=infos[*it];
+        hint=v_hint->face();
+      }
+    }
+
+    return this->number_of_vertices() - n;
+  }
+
+public:
+
+  template < class InputIterator >
+  std::ptrdiff_t
+  insert( InputIterator first,
+          InputIterator last,
+          typename boost::enable_if<
+            boost::is_convertible<
+              typename internal::Get_iterator_value_type< InputIterator >::type,
+              std::pair<Point,typename internal::Info_check<typename Tds::Vertex>::type>
+            >
+          >::type* =NULL
+  )
+  {
+    return insert_with_info< std::pair<Point,typename internal::Info_check<typename Tds::Vertex>::type> >(first,last);
+  }
+
+  template <class  InputIterator_1,class InputIterator_2>
+  std::ptrdiff_t
+  insert( boost::zip_iterator< boost::tuple<InputIterator_1,InputIterator_2> > first,
+          boost::zip_iterator< boost::tuple<InputIterator_1,InputIterator_2> > last,
+          typename boost::enable_if<
+            boost::mpl::and_<
+              boost::is_convertible< typename std::iterator_traits<InputIterator_1>::value_type, Point >,
+              boost::is_convertible< typename std::iterator_traits<InputIterator_2>::value_type, typename internal::Info_check<typename Tds::Vertex>::type >
+            >
+          >::type* =NULL
+  )
+  {
+    return insert_with_info< boost::tuple<Point,typename internal::Info_check<typename Tds::Vertex>::type> >(first,last);
+  }
+#endif //CGAL_TRIANGULATION_2_DONT_INSERT_RANGE_OF_POINTS_WITH_INFO
+
+
+  template <class IndicesIterator>
+  std::size_t insert_constraints( const std::vector<Point>& points,
+                                  IndicesIterator indices_first,
+                                  IndicesIterator indices_beyond )
+  {
+    typedef std::vector<std::ptrdiff_t> Vertex_indices;
+    typedef std::vector<Vertex_handle> Vertices;
+
+    Vertex_indices vertex_indices;
+    vertex_indices.resize(points.size());
+
+    std::copy(boost::counting_iterator<std::ptrdiff_t>(0),
+              boost::counting_iterator<std::ptrdiff_t>(points.size()),
+              std::back_inserter(vertex_indices));
+
+    size_type n = this->number_of_vertices();
+    Spatial_sort_traits_adapter_2<Gt, const Point*> sort_traits(&(points[0]));
+
+    spatial_sort(vertex_indices.begin(), vertex_indices.end(), sort_traits);
+
+    Vertices vertices;
+    vertices.resize(points.size());
+
+    Face_handle hint;
+    for(typename Vertex_indices::const_iterator
+          it_pti = vertex_indices.begin(), end = vertex_indices.end();
+          it_pti != end; ++it_pti)
+    {
+      vertices[*it_pti] = insert(points[*it_pti], hint);
+      hint=vertices[*it_pti]->face();
+    }
+
+    for(IndicesIterator it_cst=indices_first, end=indices_beyond;
+        it_cst!=end; ++it_cst)
+    {
+      Vertex_handle v1 = vertices[it_cst->first];
+      Vertex_handle v2 = vertices[it_cst->second];
+      if(v1 != v2) insert_constraint(v1, v2);
+    }
+
+    return this->number_of_vertices() - n;
+  }
+
+  template <class PointIterator, class IndicesIterator>
+  std::size_t insert_constraints(PointIterator points_first,
+                                 PointIterator points_beyond,
+                                 IndicesIterator indices_first,
+                                 IndicesIterator indices_beyond)
+  {
+    std::vector<Point> points(points_first, points_beyond);
+    return insert_constraints(points, indices_first, indices_beyond);
+  }
+
+  template <class Segment_2>
+  static const Point& get_source(const Segment_2& segment){
+    return segment.source();
+  }
+  template <class Segment_2>
+  static const Point& get_target(const Segment_2& segment){
+    return segment.target();
+  }
+
+  static const Point& get_source(const Constraint& cst){
+    return cst.first;
+  }
+  static const Point& get_target(const Constraint& cst){
+    return cst.second;
+  }
+
+  template <class ConstraintIterator>
+  std::size_t insert_constraints(ConstraintIterator first,
+                                 ConstraintIterator beyond)
+  {
+    std::vector<Point> points;
+    for (ConstraintIterator s_it=first; s_it!=beyond; ++s_it)
+    {
+      points.push_back( get_source(*s_it) );
+      points.push_back( get_target(*s_it) );
+    }
+
+    std::vector< std::pair<std::size_t, std::size_t> > segment_indices;
+    std::size_t nb_segments = points.size() / 2;
+    segment_indices.reserve( nb_segments );
+    for (std::size_t k=0; k < nb_segments; ++k)
+      segment_indices.push_back( std::make_pair(2*k,2*k+1) );
+
+    return insert_constraints( points,
+                               segment_indices.begin(),
+                               segment_indices.end() );
+  }
+
   template <class OutputItFaces, class OutputItBoundaryEdges> 
   std::pair<OutputItFaces,OutputItBoundaryEdges>
   get_conflicts_and_boundary(const Point  &p, 
