@@ -166,7 +166,7 @@ namespace CGAL {
     Self& operator++()
     {
       CGAL_assertion(this->cont());
-      this->set_current_dart(NULL);
+      this->set_current_dart(this->mmap->null_handle);
       this->mprev_op = OP_END;
       return *this;
     }
@@ -230,7 +230,7 @@ namespace CGAL {
     {
       CGAL_assertion(this->cont());
 
-      if (mfirst_dir && (*this)->is_free(0))
+      if (mfirst_dir && this->mmap->is_free(*this, 0))
       {
         this->set_current_dart(this->mfirst_dart);
         mfirst_dir = false;
@@ -243,25 +243,25 @@ namespace CGAL {
 
       if (mfirst_dir)
       {
-        CGAL_assertion(!(*this)->is_free(0));
-        this->set_current_dart((*this)->beta(0));
+        CGAL_assertion(!this->mmap->is_free(*this, 0));
+        this->set_current_dart(this->mmap->beta(*this, 0));
 
         if ((*this)==this->mfirst_dart)
         {
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
           this->mprev_op = OP_END;
         }
       }
       else
       {
-        if ((*this)->is_free(1))
+        if (this->mmap->is_free(*this, 1))
         {
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
           this->mprev_op = OP_END;
         }
         else
         {
-          this->set_current_dart((*this)->beta(1));
+          this->set_current_dart(this->mmap->beta(*this, 1));
           this->mprev_op = OP_BETAI_INV;
         }
       }
@@ -320,7 +320,7 @@ namespace CGAL {
     {
       CGAL_assertion(this->cont());
 
-      if (mfirst_dir && (*this)->is_free(1))
+      if (mfirst_dir && this->mmap->is_free(*this, 1))
       {
         this->set_current_dart(this->mfirst_dart);
         mfirst_dir = false;
@@ -333,25 +333,25 @@ namespace CGAL {
 
       if (mfirst_dir)
       {
-        CGAL_assertion(!(*this)->is_free(1));
-        this->set_current_dart((*this)->beta(1));
+        CGAL_assertion(!this->mmap->is_free(*this, 1));
+        this->set_current_dart(this->mmap->beta(*this, 1));
 
         if ((*this)==this->mfirst_dart)
         {
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
           this->mprev_op = OP_END;
         }
       }
       else
       {
-        if ((*this)->is_free(0))
+        if (this->mmap->is_free(*this, 0))
         {
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
           this->mprev_op = OP_END;
         }
         else
         {
-          this->set_current_dart((*this)->beta(0));
+          this->set_current_dart(this->mmap->beta(*this, 0));
           this->mprev_op = OP_BETAI_INV;
         }
       }
@@ -400,14 +400,14 @@ namespace CGAL {
     Self& operator++()
     {
       CGAL_assertion(this->cont());
-      if ((*this)!=this->mfirst_dart || (*this)->is_free(Bi))
+      if ((*this)!=this->mfirst_dart || this->mmap->is_free(*this, Bi))
       {
-        this->set_current_dart(NULL);
+        this->set_current_dart(this->mmap->null_handle);
         this->mprev_op = OP_END;
       }
       else
       {
-        this->set_current_dart((*this)->beta(Bi));
+        this->set_current_dart(this->mmap->beta(*this, Bi));
         this->mprev_op = OP_BETAI;
       }
       return *this;
@@ -468,45 +468,45 @@ namespace CGAL {
 
       if (mcurdart==0)
       {
-        if (!(*this)->is_free(Bi))
+        if (!this->mmap->is_free(*this, Bi))
         {
-          this->set_current_dart((*this)->beta(Bi));
+          this->set_current_dart(this->mmap->beta(*this, Bi));
           this->mprev_op = OP_BETAI;
           mcurdart=1;
         }
         else
         {
-          if (!(*this)->is_free(Bi+delta))
+          if (!this->mmap->is_free(*this, Bi+delta))
           {
-            this->set_current_dart((*this)->beta(Bi+delta));
+            this->set_current_dart(this->mmap->beta(*this, Bi+delta));
             this->mprev_op = OP_BETAJ;
             mcurdart=3;
           }
           else
           {
             this->mprev_op = OP_END;
-            this->set_current_dart(NULL);
+            this->set_current_dart(this->mmap->null_handle);
           }
         }
       }
       else if (mcurdart==1)
       {
-        if (!(*this)->is_free(Bi+delta))
+        if (!this->mmap->is_free(*this, Bi+delta))
         {
-          this->set_current_dart((*this)->beta(Bi+delta));
+          this->set_current_dart(this->mmap->beta(*this, Bi+delta));
           this->mprev_op = OP_BETAJ;
           mcurdart=2;
         }
         else
         {
           this->mprev_op = OP_END;
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
         }
       }
       else if (mcurdart==2)
       {
-        CGAL_assertion(!(*this)->is_free(Bi));
-        this->set_current_dart((*this)->beta(Bi));
+        CGAL_assertion(!this->mmap->is_free(*this, Bi));
+        this->set_current_dart(this->mmap->beta(*this, Bi));
         this->mprev_op = OP_BETAI;
         mcurdart=1;
       }
@@ -514,7 +514,7 @@ namespace CGAL {
       {
         CGAL_assertion (mcurdart==3);
         this->mprev_op = OP_END;
-        this->set_current_dart(NULL);
+        this->set_current_dart(this->mmap->null_handle);
       }
 
       return *this;
@@ -623,7 +623,8 @@ namespace CGAL {
       mexist_betaj(false),
       mprev_betaj(false),
       mfirst_border(true)
-    { if (adart!=NULL) mexist_betaj=!adart->is_free(delta); }
+    { if (adart!=this->mmap->null_handle)
+        mexist_betaj=!this->mmap->is_free(adart, delta); }
     
     /// Main constructor.
     CMap_dart_iterator_basic_of_two_beta(Map& amap, Dart_handle adart, 
@@ -633,7 +634,8 @@ namespace CGAL {
       mexist_betaj(false),
       mprev_betaj(false),
       mfirst_border(true)
-    { if (adart!=NULL) mexist_betaj=!adart->is_free(delta); }
+    { if (adart!=this->mmap->null_handle)
+        mexist_betaj=!this->mmap->is_free(adart, delta); }
 
     /// Prefix ++ operator.
     Self& operator++()
@@ -643,7 +645,7 @@ namespace CGAL {
       {
         mprev_betaj = true;
         mfirst_border = ! mfirst_border;
-        this->set_current_dart((*this)->beta(delta));
+        this->set_current_dart(this->mmap->beta(*this, delta));
         this->mprev_op = OP_BETAJ;
       }
       else
@@ -652,7 +654,7 @@ namespace CGAL {
         ++mit;
         this->mprev_op = mit.prev_operation();
         if ( !mit.cont() ) 
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
         else
         {          
           if ( !mfirst_border ) 
@@ -721,7 +723,8 @@ namespace CGAL {
       mexist_betaj(false),
       mprev_betaj(false),
       mfirst_border(true)
-    { if (adart!=NULL) mexist_betaj=!adart->is_free(1+delta); }
+    { if (adart!=this->mmap->null_handle)
+        mexist_betaj=!this->mmap->is_free(adart, 1+delta); }
     
     /// Main constructor.
     CMap_dart_iterator_basic_of_two_beta(Map& amap, Dart_handle adart, 
@@ -731,7 +734,8 @@ namespace CGAL {
       mexist_betaj(false),
       mprev_betaj(false),
       mfirst_border(true)
-    { if (adart!=NULL) mexist_betaj=!adart->is_free(1+delta); }
+    { if (adart!=this->mmap->null_handle)
+        mexist_betaj=!this->mmap->is_free(adart, 1+delta); }
 
     /// Prefix ++ operator.
     Self& operator++()
@@ -741,7 +745,7 @@ namespace CGAL {
       {
         mprev_betaj = true;
         mfirst_border = ! mfirst_border;
-        this->set_current_dart((*this)->beta(1+delta));
+        this->set_current_dart(this->mmap->beta(*this, 1+delta));
         this->mprev_op = OP_BETAJ;
       }
       else
@@ -750,11 +754,11 @@ namespace CGAL {
         ++mit;
         this->mprev_op = mit.prev_operation();
         if ( !mit.cont() ) 
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
         else
         {          
           if ( !mfirst_border ) 
-            this->set_current_dart(mit->beta(1+delta));
+            this->set_current_dart(this->mmap->beta(mit, 1+delta));
           else
             this->set_current_dart(mit);
         }
@@ -844,51 +848,51 @@ namespace CGAL {
       {
         if (mnext_try_betai)
         {
-          if ((*this)->is_free(Bi))
+          if (this->mmap->is_free(*this, Bi))
           {
             mfirst_dir = false;
-            if (this->mfirst_dart->is_free(Bi+1))
+            if (this->mmap->is_free(this->mfirst_dart, Bi+1))
             {
               this->mprev_op = OP_END;
-              this->set_current_dart(NULL);
+              this->set_current_dart(this->mmap->null_handle);
             }
             else
             {
-              this->set_current_dart(this->mfirst_dart->beta(Bi+1));
+              this->set_current_dart(this->mmap->beta(this->mfirst_dart, Bi+1));
               this->mprev_op = OP_JUMP;
             }
           }
           else
           {
-            this->set_current_dart((*this)->beta(Bi));
+            this->set_current_dart(this->mmap->beta(*this, Bi));
             mnext_try_betai = false;
             this->mprev_op = OP_BETAI;
           }
         }
         else
         {
-          if ((*this)->is_free(Bi+1))
+          if (this->mmap->is_free(*this, Bi+1))
           {
             mfirst_dir = false;
-            if (this->mfirst_dart->is_free(Bi+1))
+            if (this->mmap->is_free(this->mfirst_dart, Bi+1))
             {
               this->mprev_op = OP_END;
-              this->set_current_dart(NULL);
+              this->set_current_dart(this->mmap->null_handle);
             }
             else
             {
-              this->set_current_dart(this->mfirst_dart->beta(Bi+1));
+              this->set_current_dart(this->mmap->beta(this->mfirst_dart, Bi+1));
               mnext_try_betai = true;
               this->mprev_op = OP_JUMP;
             }
           }
           else
           {
-            this->set_current_dart((*this)->beta(Bi+1));
+            this->set_current_dart(this->mmap->beta(*this, Bi+1));
             if ((*this)==this->mfirst_dart)
             {
               this->mprev_op = OP_END;
-              this->set_current_dart(NULL);
+              this->set_current_dart(this->mmap->null_handle);
             }
             else
             {
@@ -902,28 +906,28 @@ namespace CGAL {
       {
         if (mnext_try_betai)
         {
-          if ((*this)->is_free(Bi))
+          if (this->mmap->is_free(*this, Bi))
           {
             this->mprev_op = OP_END;
-            this->set_current_dart(NULL);
+            this->set_current_dart(this->mmap->null_handle);
           }
           else
           {
-            this->set_current_dart((*this)->beta(Bi));
+            this->set_current_dart(this->mmap->beta(*this, Bi));
             mnext_try_betai = false;
             this->mprev_op = OP_BETAI;
           }
         }
         else
         {
-          if ((*this)->is_free(Bi+1))
+          if (this->mmap->is_free(*this, Bi+1))
           {
             this->mprev_op = OP_END;
-            this->set_current_dart(NULL);
+            this->set_current_dart(this->mmap->null_handle);
           }
           else
           {
-            this->set_current_dart((*this)->beta(Bi+1));
+            this->set_current_dart(this->mmap->beta(*this, Bi+1));
             mnext_try_betai = true;
             this->mprev_op = OP_BETAJ;
           }
@@ -1133,7 +1137,7 @@ namespace CGAL {
       { this->mprev_op = OP_POP; }
       else
       {
-        this->set_current_dart(NULL);
+        this->set_current_dart(this->mmap->null_handle);
         this->mprev_op = OP_END;
       }
       return *this;
@@ -1172,7 +1176,7 @@ namespace CGAL {
       Base(amap, adart),
       mmark_number(amark)
     {
-      if (adart!=NULL)
+      if (adart!=this->mmap->null_handle)
       {
         this->mmap->mark_null_dart(mmark_number);
         this->mmap->mark(adart, mmark_number);
@@ -1194,44 +1198,44 @@ namespace CGAL {
     {
       CGAL_assertion(mmark_number != -1);
       CGAL_assertion(this->cont());
-      Dart_handle nd = NULL;
+      Dart_handle nd = this->mmap->null_handle;
       
       for ( unsigned int k=0; k<i; ++k )
       {
         if ( this->is_unmarked((*this), k, mmark_number) )
         {
-          if (nd == NULL)
+          if (nd == this->mmap->null_handle)
           {
-            nd = (*this)->beta(k);
-            CGAL_assertion(nd!=Map::null_dart_handle);
+            nd = this->mmap->beta(*this, k);
+            CGAL_assertion(nd!=this->mmap->null_dart_handle);
             this->mprev_op = OP_BETAI;
           }
           else
           {
-            mto_treat.push((*this)->beta(k));
+            mto_treat.push(this->mmap->beta(*this, k));
           }
-          this->mmap->mark((*this)->beta(k), mmark_number);
+          this->mmap->mark(this->mmap->beta(*this, k), mmark_number);
         }
       }
       for ( unsigned int k=i+1; k<=d; ++k )
       {
         if ( this->is_unmarked((*this), k, mmark_number) )
         {
-          if (nd == NULL)
+          if (nd == this->mmap->null_handle)
           {
-            nd = (*this)->beta(k);
-            CGAL_assertion(nd!=Map::null_dart_handle);
+            nd = this->mmap->beta(*this, k);
+            CGAL_assertion(nd!=this->mmap->null_dart_handle);
             this->mprev_op = OP_BETAI;
           }
           else
           {
-            mto_treat.push((*this)->beta(k));
+            mto_treat.push(this->mmap->beta(*this, k));
           }
-          this->mmap->mark((*this)->beta(k), mmark_number);
+          this->mmap->mark(this->mmap->beta(*this, k), mmark_number);
         }
       }
 
-      if (nd == NULL)
+      if (nd == this->mmap->null_handle)
       {
         if (!mto_treat.empty())
         {
@@ -1242,7 +1246,7 @@ namespace CGAL {
         else
         {
           this->mprev_op = OP_END;
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
         }
       }
       
@@ -1284,7 +1288,7 @@ namespace CGAL {
       Base(amap, adart),
       mmark_number(amark)
     {
-      if (adart!=NULL)
+      if (adart!=this->mmap->null_handle)
       {
         this->mmap->mark(adart, mmark_number);
         this->mmap->mark_null_dart(mmark_number);
@@ -1307,39 +1311,39 @@ namespace CGAL {
       CGAL_assertion(mmark_number != -1);
       CGAL_assertion(this->cont());
 
-      Dart_handle nd = NULL;
+      Dart_handle nd = this->mmap->null_handle;
       
       for ( unsigned int k=2; k<=d; ++k )
       {
         if ( this->is_unmarked((*this), k, mmark_number) )
         {
-          if (nd == NULL)
+          if (nd == this->mmap->null_handle)
           {
-            nd = (*this)->beta(k);
-            CGAL_assertion(nd!=Map::null_dart_handle);
+            nd = this->mmap->beta(*this, k);
+            CGAL_assertion(nd!=this->mmap->null_dart_handle);
             this->mprev_op = OP_BETAI;
           }
           else
           {
-            mto_treat.push((*this)->beta(k));
+            mto_treat.push(this->mmap->beta(*this, k));
           }
-          this->mmap->mark((*this)->beta(k), mmark_number);
+          this->mmap->mark(this->mmap->beta(*this, k), mmark_number);
         }
       }
 
-      if (nd == NULL)
+      if (nd == this->mmap->null_handle)
       {
         if (!mto_treat.empty())
         {
           nd = mto_treat.front();
-          CGAL_assertion(nd!=Map::null_dart_handle);
+          CGAL_assertion(nd!=this->mmap->null_dart_handle);
           mto_treat.pop();
           this->mprev_op = OP_POP;
         }
         else
         {
           this->mprev_op = OP_END;
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
         }
       }
       
@@ -1380,7 +1384,7 @@ namespace CGAL {
                                      int amark):
       Base(amap, adart),
       mmark_number(amark)
-    { if (adart!=NULL)
+    { if (adart!=this->mmap->null_handle)
       {
         this->mmap->mark(adart, mmark_number);
         this->mmap->mark_null_dart(mmark_number);
@@ -1403,84 +1407,84 @@ namespace CGAL {
       CGAL_assertion(mmark_number != -1);
       CGAL_assertion(this->cont());
 
-      Dart_handle nd = NULL;
+      Dart_handle nd = this->mmap->null_handle;
 
       for ( unsigned int k=2; k<=d; ++k )
       {
         if ( this->is_unmarked2((*this), 0, k, mmark_number) )
         {
-          if (nd == NULL)
+          if (nd == this->mmap->null_handle)
           {
-            nd = (*this)->beta(0)->beta(k);
-            CGAL_assertion(nd!=Map::null_dart_handle);
+            nd = this->mmap->beta(*this, 0, k);
+            CGAL_assertion(nd!=this->mmap->null_dart_handle);
             this->mprev_op = OP_BETA0I;
           }
           else
           {
-            mto_treat.push((*this)->beta(0)->beta(k));
+            mto_treat.push(this->mmap->beta(*this, 0, k));
           }
-          this->mmap->mark((*this)->beta(0)->beta(k), mmark_number);                      
+          this->mmap->mark(this->mmap->beta(*this, 0, k), mmark_number);
         }
         if ( this->is_unmarked2((*this), k, 1, mmark_number) )
         {
-          if (nd == NULL)
+          if (nd == this->mmap->null_handle)
           {
-            nd = (*this)->beta(k)->beta(1); 
-            CGAL_assertion(nd!=Map::null_dart_handle);
+            nd = this->mmap->beta(*this, k, 1);
+            CGAL_assertion(nd!=this->mmap->null_dart_handle);
             this->mprev_op = OP_BETAI1;
           }
           else
           {
-            mto_treat.push((*this)->beta(k)->beta(1));
+            mto_treat.push(this->mmap->beta(*this, k, 1));
           }
-          this->mmap->mark((*this)->beta(k)->beta(1), mmark_number);
+          this->mmap->mark(this->mmap->beta(*this, k, 1), mmark_number);
         }
         for ( unsigned int l=k+1; l<=d; ++l )
         {
           if ( this->is_unmarked2((*this), k, l, mmark_number) )
           {
-            if (nd == NULL)
+            if (nd == this->mmap->null_handle)
             {
-              nd = (*this)->beta(k)->beta(l); 
-              CGAL_assertion(nd!=Map::null_dart_handle);
+              nd = this->mmap->beta(*this, k, l);
+              CGAL_assertion(nd!=this->mmap->null_dart_handle);
               this->mprev_op = OP_BETAIJ;
             }
             else
             {
-              mto_treat.push((*this)->beta(k)->beta(l));
+              mto_treat.push(this->mmap->beta(*this, k, l));
             }
-            this->mmap->mark((*this)->beta(k)->beta(l), mmark_number);
+            this->mmap->mark(this->mmap->beta(*this, k, l), mmark_number);
           }
           if ( this->is_unmarked2((*this), l, k, mmark_number) )
           {
-            if (nd == NULL)
+            if (nd == this->mmap->null_handle)
             {
-              nd = (*this)->beta(l)->beta(k); 
-              CGAL_assertion(nd!=Map::null_dart_handle);
+              nd = this->mmap->beta(*this, l, k);
+              CGAL_assertion(nd!=this->mmap->null_dart_handle);
               this->mprev_op = OP_BETAJI;
             }
             else
             {
-              mto_treat.push((*this)->beta(l)->beta(k));
+              mto_treat.push(this->mmap->beta(*this, l, k));
             }
-            this->mmap->mark((*this)->beta(l)->beta(k), mmark_number);
+            this->mmap->mark(this->mmap->beta(*this, l, k), mmark_number);
           }
         }
       }
 
-      if (nd == NULL)
+      if (nd == this->mmap->null_handle)
       {
         if (!mto_treat.empty())
         {
           nd = mto_treat.front();
-          CGAL_assertion(nd!=Map::null_dart_handle);
+          CGAL_assertion(nd!=this->mmap->null_dart_handle);
           mto_treat.pop();
           this->mprev_op = OP_POP;
         }
         else
         {
           this->mprev_op = OP_END;
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
         }
       }
       
@@ -1708,15 +1712,15 @@ namespace CGAL {
 
       if (mfirst_dir)
       {
-        this->set_current_dart((*this)->beta(0)->beta(2));
-        if ((*this)==Map::null_dart_handle)
+        this->set_current_dart(this->mmap->beta(*this, 0, 2));
+        if ((*this)==this->mmap->null_dart_handle)
         {
           mfirst_dir = false;
-          this->set_current_dart(this->mfirst_dart->beta(2)->beta(1));
-          if ((*this)==Map::null_dart_handle)
+          this->set_current_dart(this->mmap->beta(this->mfirst_dart, 2, 1));
+          if ((*this)==this->mmap->null_dart_handle)
           {
             this->mprev_op = OP_END;
-            this->set_current_dart(NULL);
+            this->set_current_dart(this->mmap->null_handle);
           }
           else
           {
@@ -1728,7 +1732,7 @@ namespace CGAL {
           if ((*this)==this->mfirst_dart)
           {
             this->mprev_op = OP_END;
-            this->set_current_dart(NULL);
+            this->set_current_dart(this->mmap->null_handle);
           }
           else
             this->mprev_op = OP_BETA0I;
@@ -1736,11 +1740,11 @@ namespace CGAL {
       }
       else
       {
-        this->set_current_dart((*this)->beta(2)->beta(1));
-        if ((*this) == Map::null_dart_handle)
+        this->set_current_dart(this->mmap->beta(*this, 2, 1));
+        if ((*this) == this->mmap->null_dart_handle)
         {
           this->mprev_op = OP_END;
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
         }
         else
           this->mprev_op = OP_BETA21;
@@ -1900,7 +1904,7 @@ namespace CGAL {
     {
       CGAL_assertion( d>=3 && d<=Map::dimension );
       CGAL_assertion( i>=3 && i<=Map::dimension );
-      if (adart!=NULL)
+      if (adart!=this->mmap->null_handle)
       {
         this->mmap->mark(adart, mmark_number);
         this->mmap->mark_null_dart(mmark_number);
@@ -1923,23 +1927,23 @@ namespace CGAL {
       CGAL_assertion(mmark_number != -1);
       CGAL_assertion(this->cont());
 
-      Dart_handle nd = NULL;
+      Dart_handle nd = this->mmap->null_handle;
 
       for ( int k=0; k<2; ++k )
       {
         if ( this->is_unmarked((*this), k, mmark_number) )
         {
-          if (nd == NULL)
+          if (nd == this->mmap->null_handle)
           {
-            nd = (*this)->beta(k);
-            CGAL_assertion(nd!=Map::null_dart_handle);
+            nd = this->mmap->beta(*this, k);
+            CGAL_assertion(nd!=this->mmap->null_dart_handle);
             this->mprev_op = OP_BETAI;
           }
           else
           {
-            mto_treat.push((*this)->beta(k));
+            mto_treat.push(this->mmap->beta(*this, k));
           }
-          this->mmap->mark((*this)->beta(k), mmark_number);
+          this->mmap->mark(this->mmap->beta(*this, k), mmark_number);
         }
       }
 
@@ -1948,21 +1952,21 @@ namespace CGAL {
         if ( k!=i-1 && k!=i && k!=i+1 && 
              this->is_unmarked((*this), k, mmark_number) )
         {
-          if (nd == NULL)
+          if (nd == this->mmap->null_handle)
           {
-            nd = (*this)->beta(k);
-            CGAL_assertion(nd!=Map::null_dart_handle);
+            nd = this->mmap->beta(*this, k);
+            CGAL_assertion(nd!=this->mmap->null_dart_handle);
             this->mprev_op = OP_BETAI;
           }
           else
           {
-            mto_treat.push((*this)->beta(k));
+            mto_treat.push(this->mmap->beta(*this, k));
           }
-          this->mmap->mark((*this)->beta(k), mmark_number);
+          this->mmap->mark(this->mmap->beta(*this, k), mmark_number);
         }
       }
 
-      if (nd == NULL)
+      if (nd == this->mmap->null_handle)
       {
         if (!mto_treat.empty())
         {
@@ -1973,7 +1977,7 @@ namespace CGAL {
         else
         {
           this->mprev_op = OP_END;
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
         }
       }
       
@@ -2022,7 +2026,7 @@ namespace CGAL {
       mmark_number(amark)
     {
       CGAL_assertion( i>=3 && i<=Map::dimension );
-      if (adart!=NULL)
+      if (adart!=this->mmap->null_handle)
       {
         this->mmap->mark(adart, mmark_number);
         this->mmap->mark_null_dart(mmark_number);
@@ -2045,23 +2049,23 @@ namespace CGAL {
       CGAL_assertion(mmark_number != -1);
       CGAL_assertion(this->cont());
 
-      Dart_handle nd = NULL;
+      Dart_handle nd = this->mmap->null_handle;
 
       for ( int k=1; k>=0; --k )
       {
         if ( this->is_unmarked((*this), k, mmark_number) )
         {
-          if (nd == NULL)
+          if (nd == this->mmap->null_handle)
           {
-            nd = (*this)->beta(k);
-            CGAL_assertion(nd!=Map::null_dart_handle);
+            nd = this->mmap->beta(*this, k);
+            CGAL_assertion(nd!=this->mmap->null_dart_handle);
             this->mprev_op = OP_BETAI;
           }
           else
           {
-            mto_treat.push((*this)->beta(k));
+            mto_treat.push(this->mmap->beta(*this, k));
           }
-          this->mmap->mark((*this)->beta(k), mmark_number);
+          this->mmap->mark(this->mmap->beta(*this, k), mmark_number);
         }
       }
       for ( int k=2; k<=d; ++k )
@@ -2069,21 +2073,21 @@ namespace CGAL {
         if ( k!=i-1 && k!=i && k!=i+1 && 
              this->is_unmarked((*this), k, mmark_number) )
         {
-          if (nd == NULL)
+          if (nd == this->mmap->null_handle)
           {
-            nd = (*this)->beta(k);
-            CGAL_assertion(nd!=Map::null_dart_handle);
+            nd = this->mmap->beta(*this, k);
+            CGAL_assertion(nd!=this->mmap->null_dart_handle);
             this->mprev_op = OP_BETAI;
           }
           else
           {
-            mto_treat.push((*this)->beta(k));
+            mto_treat.push(this->mmap->beta(*this, k));
           }
-          this->mmap->mark((*this)->beta(k), mmark_number);
+          this->mmap->mark(this->mmap->beta(*this, k), mmark_number);
         }
       }
 
-      if (nd == NULL)
+      if (nd == this->mmap->null_handle)
       {
         if (!mto_treat.empty())
         {
@@ -2094,7 +2098,7 @@ namespace CGAL {
         else
         {
           this->mprev_op = OP_END;
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
         }
       }
       
@@ -2139,7 +2143,7 @@ namespace CGAL {
                                            int amark):
       Base(amap, adart),
       mmark_number(amark)
-    { if (adart!=NULL)
+    { if (adart!=this->mmap->null_handle)
       {
         this->mmap->mark(adart, mmark_number);
         this->mmap->mark_null_dart(mmark_number);
@@ -2162,27 +2166,27 @@ namespace CGAL {
       CGAL_assertion(mmark_number != -1);
       CGAL_assertion(this->cont());
 
-      Dart_handle nd = NULL;
+      Dart_handle nd = this->mmap->null_handle;
 
       for ( unsigned int k=3; k<=d; ++k )
       {
         if ( this->is_unmarked((*this), k, mmark_number) )
         {
-          if (nd == NULL)
+          if (nd == this->mmap->null_handle)
           {
-            nd = (*this)->beta(k);
-            CGAL_assertion(nd!=Map::null_dart_handle);
+            nd = this->mmap->beta(*this, k);
+            CGAL_assertion(nd!=this->mmap->null_dart_handle);
             this->mprev_op = OP_BETAI;
           }
           else
           {
-            mto_treat.push((*this)->beta(k));
+            mto_treat.push(this->mmap->beta(*this, k));
           }
-          this->mmap->mark((*this)->beta(k), mmark_number);
+          this->mmap->mark(this->mmap->beta(*this, k), mmark_number);
         }
       }
 
-      if (nd == NULL)
+      if (nd == this->mmap->null_handle)
       {
         if (!mto_treat.empty())
         {
@@ -2193,7 +2197,7 @@ namespace CGAL {
         else
         {
           this->mprev_op = OP_END;
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
         }
       }
       
@@ -2265,7 +2269,7 @@ namespace CGAL {
                                            int amark):
       Base(amap, adart),
       mmark_number(amark)
-    { if ( adart!=NULL)
+    { if ( adart!=this->mmap->null_handle)
       {
         this->mmap->mark(adart, mmark_number);
         this->mmap->mark_null_dart(mmark_number);
@@ -2288,27 +2292,27 @@ namespace CGAL {
       CGAL_assertion(mmark_number != -1);
       CGAL_assertion(this->cont());
 
-      Dart_handle nd = NULL;
+      Dart_handle nd = this->mmap->null_handle;
 
       for ( unsigned int k=4; k<=d; ++k )
       {
         if ( this->is_unmarked((*this), k, mmark_number) )
         {
-          if (nd == NULL)
+          if (nd == this->mmap->null_handle)
           {
-            nd = (*this)->beta(k);
-            CGAL_assertion(nd!=Map::null_dart_handle);
+            nd = this->mmap->beta(*this, k);
+            CGAL_assertion(nd!=this->mmap->null_dart_handle);
             this->mprev_op = OP_BETAI;
           }
           else
           {
-            mto_treat.push((*this)->beta(k));
+            mto_treat.push(this->mmap->beta(*this, k));
           }
-          this->mmap->mark((*this)->beta(k), mmark_number);
+          this->mmap->mark(this->mmap->beta(*this, k), mmark_number);
         }
       }
 
-      if (nd == NULL)
+      if (nd == this->mmap->null_handle)
       {
         if (!mto_treat.empty())
         {
@@ -2319,7 +2323,7 @@ namespace CGAL {
         else
         {
           this->mprev_op = OP_END;
-          this->set_current_dart(NULL);
+          this->set_current_dart(this->mmap->null_handle);
         }
       }
       
