@@ -19,7 +19,7 @@
 #ifndef CGAL_GMPFI_TYPE_H
 #define CGAL_GMPFI_TYPE_H
 
-#include <CGAL/basic.h>
+#include <CGAL/config.h>
 #include <CGAL/gmp.h>
 #include <mpfr.h>
 #include <CGAL/GMP/Gmpfr_type.h>
@@ -1096,4 +1096,5 @@ Uncertain<Gmpfi> max BOOST_PREVENT_MACRO_SUBSTITUTION
 
 } // namespace CGAL
 
+
 #endif  // CGAL_GMPFI_TYPE_H
