@@ -26,9 +26,13 @@
 #ifndef CGAL_IMPLICIT_MESH_DOMAIN_3_H
 #define CGAL_IMPLICIT_MESH_DOMAIN_3_H
 
+#if defined(BOOST_MSVC)
+#  pragma warning(push)
+#  pragma warning(disable:4180) // qualifier applied to function type has no meaning; ignored
+#endif
 
 #include <CGAL/Mesh_3/Labeled_mesh_domain_3.h>
-#include <CGAL/Mesh_3/implicit_to_labeled_function_wrapper.h>
+#include <CGAL/Mesh_3/Implicit_to_labeled_function_wrapper.h>
 
 namespace CGAL {
 
@@ -60,7 +64,7 @@ public:
    * @param bounding_sphere a bounding sphere of the domain
    * @param error_bound the error bound relative to the sphere radius
    */
-  Implicit_mesh_domain_3(Function& f,
+  Implicit_mesh_domain_3(const Function& f,
                          const Sphere_3& bounding_sphere,
                          const FT& error_bound = FT(1e-3))
     : Base(Wrapper(f), bounding_sphere, error_bound)  {}
@@ -80,4 +84,8 @@ private:
 
 }  // end namespace CGAL
 
+#if defined(BOOST_MSVC)
+#  pragma warning(pop)
+#endif
+
 #endif // CGAL_IMPLICIT_MESH_DOMAIN_3_H
