@@ -26,6 +26,8 @@
 #ifndef CGAL_MESH_3_LABELED_MESH_DOMAIN_3_H
 #define CGAL_MESH_3_LABELED_MESH_DOMAIN_3_H
 
+#include <CGAL/Mesh_3/config.h>
+
 #include <CGAL/Bbox_3.h>
 
 #include <CGAL/point_generators_3.h>
@@ -201,12 +203,19 @@ public:
     template<typename Query>
     Surface_patch clip_to_segment(const Query& query) const
     {
-      const Object clipped = CGAL::intersection(query, r_domain_.bbox_);
+      typename cpp11::result_of<typename BGT::Intersect_3(Query, Iso_cuboid_3)>::type
+        clipped = CGAL::intersection(query, r_domain_.bbox_);
 
-      if ( const Segment_3* s = object_cast<Segment_3>(&clipped) )
-        return this->operator()(*s);
-      else
-        return Surface_patch();
+      if(clipped)
+#if CGAL_INTERSECTION_VERSION > 1
+        if(const Segment_3* s = boost::get<Segment_3>(&*clipped))
+          return this->operator()(*s);
+#else
+        if(const Segment_3* s = object_cast<Segment_3>(&clipped))
+          return this->operator()(*s);
+#endif
+        
+      return Surface_patch();
     }
 
   private:
@@ -235,7 +244,9 @@ public:
 
     Intersection operator()(const Segment_3& s) const
     {
+#ifndef CGAL_MESH_3_NO_LONGER_CALLS_DO_INTERSECT_3
       CGAL_precondition(r_domain_.do_intersect_surface_object()(s));
+#endif // NOT CGAL_MESH_3_NO_LONGER_CALLS_DO_INTERSECT_3
       return this->operator()(s.source(),s.target());
     }
 
@@ -283,6 +294,14 @@ public:
         return Intersection();
       }
 
+      // Construct the surface patch index and index from the values at 'a'
+      // and 'b'. Even if the bissection find out a different pair of
+      // values, the reported index will be constructed from the initial
+      // values.
+      const Surface_patch_index sp_index =
+        r_domain_.make_surface_index(value_at_p1, value_at_p2);
+      const Index index = r_domain_.index_from_surface_patch_index(sp_index);
+
       // Else lets find a point (by bisection)
       // Bisection ends when the point is near than error bound from surface
       while(true)
@@ -290,11 +309,8 @@ public:
         // If the two points are enough close, then we return midpoint
         if ( squared_distance(p1, p2) < r_domain_.squared_error_bound_ )
         {
-          return Intersection(
-              mid,
-              r_domain_.index_from_surface_patch_index(
-                  r_domain_.make_surface_index(value_at_p1, value_at_p2)),
-              2);
+          CGAL_assertion(value_at_p1 != value_at_p2);
+          return Intersection(mid, index, 2);
         }
 
         // Else we must go on
@@ -322,12 +338,19 @@ public:
     template<typename Query>
     Intersection clip_to_segment(const Query& query) const
     {
-      const Object clipped = CGAL::intersection(query, r_domain_.bbox_);
+      typename cpp11::result_of<typename BGT::Intersect_3(Query, Iso_cuboid_3)>::type
+        clipped = CGAL::intersection(query, r_domain_.bbox_);
 
-      if ( const Segment_3* s = object_cast<Segment_3>(&clipped) )
-        return this->operator()(*s);
-      else
-        return Intersection();
+      if(clipped)
+#if CGAL_INTERSECTION_VERSION > 1
+        if(const Segment_3* s = boost::get<Segment_3>(&*clipped))
+          return this->operator()(*s);
+#else
+        if(const Segment_3* s = object_cast<Segment_3>(&clipped))
+          return this->operator()(*s);
+#endif
+      
+      return Intersection();
     }
 
   private:
