@@ -26,8 +26,10 @@
 #ifndef CGAL_MESH_3_MESH_SURFACE_CELL_BASE_3_H
 #define CGAL_MESH_3_MESH_SURFACE_CELL_BASE_3_H
 
+#include <CGAL/Mesh_3/config.h>
 
 #include <CGAL/Regular_triangulation_cell_base_3.h>
+#include <CGAL/Mesh_3/io_signature.h>
 
 #ifdef _MSC_VER
 // Kill warning "C4351: new behavior: elements of array
@@ -185,6 +187,13 @@ public:
   // End backward Compatibility
   // -----------------------------------
 
+  static
+  std::string io_signature()
+  {
+    return
+      Get_io_signature<Cb>()() + "+(" +
+      Get_io_signature<Surface_patch_index>()() + ")[4]";
+  }
 private:
   /// Stores surface_index for each facet of the cell
   Surface_patch_index surface_index_table_[4];
@@ -233,7 +242,7 @@ operator<<(std::ostream &os,
     if(is_ascii(os))
       os << ' ' << c.surface_patch_index(i);
     else
-      write(os, static_cast<int>(c.surface_patch_index(i)));
+      write(os, c.surface_patch_index(i));
   }
   return os;
 }
