@@ -27,6 +27,10 @@
 #ifndef CGAL_MESH_3_MESHER_3_H
 #define CGAL_MESH_3_MESHER_3_H
 
+#include <CGAL/Mesh_3/config.h>
+
+#include <CGAL/Mesh_3/Dump_c3t3.h>
+
 #include<CGAL/Mesh_3/Refine_facets_3.h>
 #include<CGAL/Mesh_3/Refine_cells_3.h>
 #include <CGAL/Mesh_3/Refine_tets_visitor.h>
@@ -106,7 +110,7 @@ public:
   ~Mesher_3() { }
   
   /// Launch mesh refinement
-  double refine_mesh();
+  double refine_mesh(std::string dump_after_refine_surface_prefix = "");
 
   /// Debug
   std::string debug_info() const;
@@ -187,7 +191,7 @@ Mesher_3<C3T3,MC,MD>::Mesher_3(C3T3& c3t3,
 
 template<class C3T3, class MC, class MD>
 double
-Mesher_3<C3T3,MC,MD>::refine_mesh()
+Mesher_3<C3T3,MC,MD>::refine_mesh(std::string dump_after_refine_surface_prefix)
 {
   CGAL::Timer timer;
   timer.start();
@@ -206,10 +210,12 @@ Mesher_3<C3T3,MC,MD>::refine_mesh()
   // refined again if it is encroached)
   facets_visitor_.activate();
 
+  dump_c3t3(r_c3t3_, dump_after_refine_surface_prefix);
+
   // Then scan volume and refine it
   cells_mesher_.scan_triangulation();
   cells_mesher_.refine(cells_visitor_);
-#else
+#else // if defined(CGAL_MESH_3_VERBOSE)
   std::cerr << "Start surface scan...";
   facets_mesher_.scan_triangulation();
   std::cerr << "end scan. [Bad facets:" << facets_mesher_.size() << "]";
@@ -248,6 +254,7 @@ Mesher_3<C3T3,MC,MD>::refine_mesh()
   nbsteps = 0;
   
   facets_visitor_.activate();
+  dump_c3t3(r_c3t3_, dump_after_refine_surface_prefix);
   std::cerr << "Start volume scan...";
   cells_mesher_.scan_triangulation();
   std::cerr << "end scan. [Bad tets:" << cells_mesher_.size() << "]";
