@@ -1,4 +1,5 @@
 // Copyright (c) 2009-2010 INRIA Sophia-Antipolis (France).
+// Copyright (c) 2010-2013 GeometryFactory Sarl (France)
 // All rights reserved.
 //
 // This file is part of CGAL (www.cgal.org).
@@ -16,7 +17,7 @@
 // $Id$
 //
 //
-// Author(s)     : Stephane Tayeb
+// Author(s)     : Stephane Tayeb, Laurent Rineau
 //
 //******************************************************************************
 // File Description : 
@@ -25,13 +26,24 @@
 #ifndef CGAL_MESH_3_PROTECT_EDGES_SIZING_FIELD_H
 #define CGAL_MESH_3_PROTECT_EDGES_SIZING_FIELD_H
 
-#include <CGAL/Delaunay_triangulation_3.h>
+#include <CGAL/Mesh_3/config.h>
 
-namespace {
+#include <CGAL/Delaunay_triangulation_3.h>
+#include <CGAL/Mesh_3/io_signature.h>
+#ifndef CGAL_NO_ASSERTIONS
+#  include <boost/math/special_functions/next.hpp> // for float_prior
+#endif
+
+namespace CGAL {
+namespace Mesh_3 {
+namespace internal {
   const double min_intersection_factor = .4; // (1-alpha)
   const double weight_modifier = .81; //0.9025;//0.81;
   const double distance_divisor = 2.1;
-}
+  const int max_nb_vertices_to_reevaluate_size = 10;
+} // end namespace internal
+} // end namespace Mesh_3
+} // end namespace CGAL
 
 #include <cmath>
 #include <algorithm>
@@ -47,11 +59,25 @@ namespace {
 #include <CGAL/enum.h>
 #include <CGAL/iterator.h>
 #include <CGAL/number_utils.h>
+#include <CGAL/squared_distance_3.h>
+
+#include <fstream>
 
 namespace CGAL {
 
 namespace Mesh_3 {
 
+template <typename C3t3>
+void debug_dump_c3t3(const std::string filename, const C3t3& c3t3)
+{
+  std::cerr << "Dump current mesh to " << filename << std::endl;
+  std::ofstream out(filename.c_str(), 
+                    std::ios_base::out|std::ios_base::binary);
+  out << "binary CGAL c3t3 " << CGAL::Get_io_signature<C3t3>()() << "\n";
+  CGAL::set_binary_mode(out);
+  out << c3t3;
+}
+
 
 template <typename C3T3, typename MeshDomain, typename SizingFunction>
 class Protect_edges_sizing_field
@@ -78,7 +104,8 @@ public:
 public:
   Protect_edges_sizing_field(C3T3& c3t3,
                              const MeshDomain& domain,
-                             SizingFunction size=SizingFunction());
+                             SizingFunction size=SizingFunction(),
+                             const FT minimal_size = FT());
   
   void operator()(const bool refine=true);
   
@@ -101,23 +128,24 @@ private:
   Vertex_handle get_vertex_corner_from_point(const Bare_point& p,
                                              const Index& p_index) const;
   
-  /// Insert point p as a curve segment point
-  Vertex_handle insert_curve_point(const Bare_point& p, const Index& p_index);
-  
   /// Insert point(p,w) into triangulation and set its dimension to \c dim and
   /// it's index to \c index.
-  /// The newly created handle is returned
+  /// The handle of the newly created vertex is returned.
   Vertex_handle insert_point(const Bare_point& p,
                              const Weight& w,
                              int dim,
-                             const Index& index);
+                             const Index& index,
+                             const bool special_ball = false);
 
   /**
    * Insert point(p,w) into triangulation and set its dimension to \c dim and
    * it's index to \c index.
-   * The newly created handle is returned
-   * This function also ensures that point(p,w) will not be inside a sphere,
-   * and that no point of the triangulation will be inside its sphere.
+   * The handle of the newly created vertex is returned.
+   * 
+   * This function also ensures that point(p,w) will not be inside a
+   * sphere, by decreasing the radius of any sphere that contains it.
+   * It also ensures that no point of the triangulation will be inside its
+   * sphere, by decreasing w.
    */
   Vertex_handle smart_insert_point(const Bare_point& p,
                                    Weight w,
@@ -150,8 +178,13 @@ private:
   bool non_adjacent_but_intersect(const Vertex_handle& va,
                                   const Vertex_handle& vb) const;
   
+  /// Returns true if balls of \c va and \c vb intersect
+  bool do_balls_intersect(const Vertex_handle& va,
+                          const Vertex_handle& vb) const;
+  
   /// Change size of the ball of vertex \c v.
-  Vertex_handle change_ball_size(const Vertex_handle& v, const FT size);
+  Vertex_handle change_ball_size(const Vertex_handle& v, const FT size,
+                                 const bool special_ball = false);
   
   
   /// Returns true if balls of v1 and v2 intersect "enough"
@@ -240,11 +273,48 @@ private:
   {
     return CGAL::sqrt(v->point().weight());
   }
-  
+
+  /// Test if a given vertex is a special protecting ball
+  /// A special ball is a protecting ball whose radius is set to the
+  /// minimal radius. Such a ball can exceptionally intersect a ball that
+  /// is on a different curve. Special balls are marked with special values
+  /// of 'in_dimension'.
+  bool is_special(const Vertex_handle&v) const
+  {
+    return v->is_special();
+  }
+
+  /// Set to a negative dimension to mark this ball as a special one.
+  void set_special(const Vertex_handle&v) const
+  {
+    v->set_special();
+  }
+
+  /// Returns the real dimension of the vertex
+  int get_dimension(const Vertex_handle& v) const
+  {
+    return c3t3_.in_dimension(v);
+  }
+
+  /// Query the sizing field and returns its value at the point p, or
+  /// minimal_size if the later is greater.
+  FT query_size(const Bare_point& p, int dim, const Index& index) const
+  {
+    if(dim < 0) dim = -1 - dim; // Convert the dimension if it was set to a
+                                // negative value (marker for special balls).
+    const FT s = size_(p, dim, index);
+    // if(minimal_size_ != FT() && s < minimal_size_) 
+    //   return minimal_size_;
+    // else
+      return s;
+  }
+
 private:
   C3T3& c3t3_;
   const MeshDomain& domain_;
   SizingFunction size_;
+  FT minimal_size_;
+  Weight minimal_weight_;
   std::set<Curve_segment_index> treated_edges_;
   std::set<Vertex_handle> unchecked_vertices_;
 };
@@ -252,10 +322,13 @@ private:
 
 template <typename C3T3, typename MD, typename Sf>
 Protect_edges_sizing_field<C3T3, MD, Sf>::
-Protect_edges_sizing_field(C3T3& c3t3, const MD& domain, Sf size)
+Protect_edges_sizing_field(C3T3& c3t3, const MD& domain, 
+                           Sf size, const FT minimal_size)
   : c3t3_(c3t3)
   , domain_(domain)
   , size_(size)
+  , minimal_size_(minimal_size)
+  , minimal_weight_(CGAL::square(minimal_size))
 {
 }
 
@@ -266,7 +339,10 @@ Protect_edges_sizing_field<C3T3, MD, Sf>::
 operator()(const bool refine)
 {
 #ifdef CGAL_MESH_3_VERBOSE
-  std::cerr << "Inserting protection balls..." << std::endl;
+  std::cerr << "Inserting protection balls..." << std::endl
+            << "  refine_balls = " << std::boolalpha << refine << std::endl
+            << "  min_balls_radius = " << minimal_size_ << std::endl
+            << "  min_balls_weight = " << minimal_weight_ << std::endl;
 #endif
   
   // Insert 0-dimensional features
@@ -291,9 +367,11 @@ operator()(const bool refine)
     std::cerr << "refine_balls() done. Nb of points in triangulation: "
               << c3t3_.triangulation().number_of_vertices() << std::endl;
 #endif
-    CGAL_assertion(c3t3_.is_valid());
-  }
+    CGAL_assertion(minimal_size_ > 0 || c3t3_.is_valid());
+ }
   
+  // debug_dump_c3t3("dump-mesh-after-protect_edges.binary.cgal", c3t3_);
+
 #ifdef CGAL_MESH_3_VERBOSE
   std::cerr << std::endl;
 #endif
@@ -329,7 +407,7 @@ insert_corners()
     Index p_index = domain_.index_from_corner_index(cit->first);
     
     // Get weight (ball radius is given by size_ function)
-    FT w = CGAL::square(size_(p, 0, p_index));
+    FT w = CGAL::square(query_size(p, 0, p_index));
 
     // the following lines ensure that the weight w is small enough so that
     // corners balls do not intersect
@@ -364,7 +442,15 @@ insert_corners()
     
     // Insert corner with ball (dim is zero because p is a corner)
     Vertex_handle v = smart_insert_point(p, w, 0, p_index);
+    CGAL_assertion(v != Vertex_handle());
+
+    // As C3t3::add_to_complex modifies the 'in_dimension' of the vertex,
+    // we need to backup and re-set the 'is_special' marker after.
+    const bool special_ball = is_special(v);
     c3t3_.add_to_complex(v,cit->first);
+    if(special_ball) {
+      set_special(v);
+    }
   }
 }
 
@@ -372,26 +458,54 @@ insert_corners()
 template <typename C3T3, typename MD, typename Sf>
 typename Protect_edges_sizing_field<C3T3, MD, Sf>::Vertex_handle
 Protect_edges_sizing_field<C3T3, MD, Sf>::
-insert_point(const Bare_point& p, const Weight& w, int dim, const Index& index)
+insert_point(const Bare_point& p, const Weight& w, int dim, const Index& index,
+             const bool special_ball /* = false */)
 {
+  using CGAL::Mesh_3::internal::weight_modifier;
+
+  if(dim < 0) dim = -1 - dim; // Convert the dimension if it was set to a
+                              // negative value (marker for special balls).
+
   typedef typename Tr::size_type size_type;
   CGAL_USE_TYPE(size_type);
   
   // Insert point
   CGAL_assertion_code(size_type nb_vertices_before = c3t3_.triangulation().number_of_vertices());
-  Vertex_handle v = c3t3_.triangulation().insert(Weighted_point(p,w*weight_modifier));
+
+  typename Tr::Locate_type lt;
+  int li, lj;
+  const typename Tr::Cell_handle ch = c3t3_.triangulation().locate(p, lt, li, lj);
+  Vertex_handle v = c3t3_.triangulation().insert(Weighted_point(p,w*weight_modifier),
+                                                 lt, ch, li, lj);
   
   // If point insertion created an hidden ball, fail
   CGAL_assertion ( Vertex_handle() != v );
-  CGAL_assertion ( c3t3_.triangulation().number_of_vertices() == (nb_vertices_before+1) );
+  CGAL_assertion ( lt == Tr::VERTEX ||
+                   c3t3_.triangulation().number_of_vertices() == (nb_vertices_before+1) );
 
-#ifdef PROTECTION_DEBUG
-  std::cerr << "Insertion of protecting ball "
-            << Weighted_point(p,w*weight_modifier)
-            << " on curve #" << index << std::endl;
-#endif
-  
-  c3t3_.set_dimension(v,dim);
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+  std::cerr << "Insertion of ";
+  if(special_ball) std::cerr << "SPECIAL ";
+  std::cerr << "protecting ball "
+            << Weighted_point(p,w*weight_modifier);
+  switch(dim) {
+  case 0:
+    std::cerr << " on corner #";
+    break;
+  case 1:
+    std::cerr << " on curve #";
+    break;
+  default:
+    std::cerr << " ERROR dim=" << dim << " index=";
+  }
+  std::cerr  << index << std::endl;
+  if(v == Vertex_handle()) std::cerr << "  HIDDEN!\n";
+  std::cerr << "The weight was " << w << std::endl;
+#endif // CGAL_MESH_3_PROTECTION_DEBUG
+
+  c3t3_.set_dimension(v, dim);
+  if(special_ball)
+    set_special(v);
   c3t3_.set_index(v,index);
   
   unchecked_vertices_.insert(v);
@@ -405,67 +519,130 @@ typename Protect_edges_sizing_field<C3T3, MD, Sf>::Vertex_handle
 Protect_edges_sizing_field<C3T3, MD, Sf>::
 smart_insert_point(const Bare_point& p, Weight w, int dim, const Index& index)
 {
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+  std::cerr << "smart_insert_point( (" << p 
+            << "), w=" << w
+            << ", dim=" << dim
+            << ", index=" << index << ")\n";
+#endif
   const Tr& tr = c3t3_.triangulation();
   typename Gt::Compute_squared_distance_3 sq_distance =
     tr.geom_traits().compute_squared_distance_3_object();
   
-  bool add_handle_to_unchecked = false;
+  bool add_handle_to_unchecked = false; /// add or not the new vertex to
+                                        /// the set 'unchecked_vertices'
+  bool insert_a_special_ball = false; /// will be passed to the function
+                                      /// this->insert_point
   
   if ( tr.dimension() > 2 ) 
   {
     // Check that new point will not be inside a power sphere
-    Cell_handle ch = tr.locate(p);
+
+    typename Tr::Locate_type lt;
+    int li, lj;
+    Cell_handle ch = tr.locate(p, lt, li, lj);
     Vertex_handle nearest_vh = tr.nearest_power_vertex(p, ch);
     FT sq_d = sq_distance(p, nearest_vh->point().point());
-    CGAL_assertion( sq_d > 0 );
-    
-    while ( nearest_vh->point().weight() > sq_d )
+    while ( nearest_vh->point().weight() > sq_d &&
+            ! is_special(nearest_vh) )
     {
+      CGAL_assertion( minimal_size_ >0 || sq_d > 0 );
+
+      bool special_ball = false;
+      if(minimal_weight_ != Weight() && sq_d < minimal_weight_) {
+        sq_d = minimal_weight_;
+        w = minimal_weight_;
+        special_ball = true;
+        insert_a_special_ball = true;
+      }
+
       // Adapt size
-      Vertex_handle new_vh = change_ball_size(nearest_vh, CGAL::sqrt(sq_d));
-      ch = tr.locate(p,new_vh);
+      Vertex_handle new_vh = change_ball_size(nearest_vh, CGAL::sqrt(sq_d),
+                                              special_ball);
+      ch = tr.locate(p, lt, li, lj, new_vh);
       
       // Iterate
       nearest_vh = tr.nearest_power_vertex(p, ch);
       sq_d = sq_distance(p, nearest_vh->point().point());
-      CGAL_assertion( sq_d > 0 );
+    }
+
+    if( is_special(nearest_vh) && nearest_vh->point().weight() > sq_d )
+    {
+      w = minimal_weight_;
+      insert_a_special_ball = true;
     }
 
     // Change w in order to be sure that no existing point will be included
     // in (p,w)
-    std::vector<Cell_handle> cells_in_conflicts;
     std::set<Vertex_handle> vertices_in_conflict_zone;
-    tr.find_conflicts(Weighted_point(p, w), ch,
-                      CGAL::Emptyset_iterator(),
-                      std::back_inserter(cells_in_conflicts),
-                      CGAL::Emptyset_iterator());
+    { // fill vertices_in_conflict_zone
+      std::vector<Cell_handle> cells_in_conflicts;
+      tr.find_conflicts(Weighted_point(p, w), ch,
+                        CGAL::Emptyset_iterator(),
+                        std::back_inserter(cells_in_conflicts),
+                        CGAL::Emptyset_iterator());
 
-    for(typename std::vector<Cell_handle>::const_iterator 
-          it = cells_in_conflicts.begin(),
-          end = cells_in_conflicts.end(); it != end; ++it) 
-    {
-      for(int i = 0, d = tr.dimension(); i <= d; ++i) {
-        vertices_in_conflict_zone.insert((*it)->vertex(i));
+      for(typename std::vector<Cell_handle>::const_iterator 
+            it = cells_in_conflicts.begin(),
+            end = cells_in_conflicts.end(); it != end; ++it) 
+      {
+        for(int i = 0, d = tr.dimension(); i <= d; ++i) {
+          const Vertex_handle v = (*it)->vertex(i);
+          if( ! c3t3_.triangulation().is_infinite(v) ) {
+            vertices_in_conflict_zone.insert(v);
+          }
+        }
       }
     }
     FT min_sq_d = w;
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+    typename Tr::Point nearest_point;
+#endif
     for(typename std::set<Vertex_handle>::const_iterator 
           it = vertices_in_conflict_zone.begin(),
           end = vertices_in_conflict_zone.end(); it != end ; ++it )
     {
-      min_sq_d = (std::min)(min_sq_d, sq_distance(p, (*it)->point().point()));
+      const FT sq_d = sq_distance(p, (*it)->point().point());
+      if(minimal_weight_ != Weight() && sq_d < minimal_weight_) {
+        insert_a_special_ball = true;
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+        nearest_point = (*it)->point();
+#endif
+        min_sq_d = minimal_weight_;
+        if( ! is_special(*it) ) {
+          ch = change_ball_size(*it, minimal_size_, true)->cell(); // special ball
+        }
+      } else {
+        if(sq_d < min_sq_d) {
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+          nearest_point = (*it)->point();
+#endif
+          min_sq_d = sq_d;
+        }
+      }
     }
 
     if ( w > min_sq_d )
     { 
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+      std::cerr << "smart_insert_point: weight " << w
+                << " reduced to " << min_sq_d
+                << "\n (near existing point: " << nearest_point << " )\n";
+#endif
       w = min_sq_d;
       add_handle_to_unchecked = true;
     }
-    CGAL_assertion_code(std::vector<Vertex_handle> hidden_vertices;);
-    CGAL_assertion_code(tr.vertices_inside_conflict_zone(Weighted_point(p, w),
-                                                         ch,
-                                                         std::back_inserter(hidden_vertices)));
-    CGAL_assertion(hidden_vertices.empty());
+
+    if(lt != Tr::VERTEX) {
+      using CGAL::Mesh_3::internal::weight_modifier;
+      CGAL_assertion_code(std::vector<Vertex_handle> hidden_vertices;);
+      CGAL_assertion_code(ch = tr.locate(p, lt, li, lj, ch););
+      CGAL_assertion_code(tr.vertices_inside_conflict_zone(Weighted_point(p, w*weight_modifier),
+                                                           ch,
+                                                           std::back_inserter(hidden_vertices)));
+
+      CGAL_assertion(hidden_vertices.empty());
+    }
   }
   else // tr.dimension() <= 2
   {
@@ -480,34 +657,62 @@ smart_insert_point(const Bare_point& p, Weight w, int dim, const Index& index)
         FT sq_d = sq_distance(p, it->point().point());
         if ( it->point().weight() > sq_d )
         { 
-          change_ball_size(it, CGAL::sqrt(sq_d));
-          restart = true;
+          bool special_ball = false;
+          if(minimal_weight_ != Weight() && sq_d > minimal_weight_) {
+            sq_d = minimal_weight_;
+            w = minimal_weight_;
+            special_ball = true;
+            insert_a_special_ball = true;
+          }
+          if( ! is_special(it) ) { 
+            change_ball_size(it, CGAL::sqrt(sq_d), special_ball);
+            restart = true;
+          }
           break;
         }
       }
     }
     
+    FT min_sq_d = w;
+    typename Tr::Point nearest_point;
     // Change w in order to be sure that no existing point will be included
     // in (p,w)
     for ( typename Tr::Finite_vertices_iterator it = tr.finite_vertices_begin(),
          end = tr.finite_vertices_end() ; it != end ; ++it )
     {
       FT sq_d = sq_distance(p, it->point().point());
-      if ( w > sq_d )
-      { 
-        w = sq_d;
-        add_handle_to_unchecked = true;
+      if(sq_d < min_sq_d) {
+        min_sq_d = sq_d;
+        nearest_point = it->point();
       }
     }    
+    if ( w > min_sq_d )
+    { 
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+      std::cerr << "smart_insert_point: weight " << w
+                << " reduced to " << min_sq_d
+                << "\n (near existing point: " << nearest_point << " )\n";
+#endif
+      w = min_sq_d;
+      add_handle_to_unchecked = true;
+    }
   }
 
-  FT w_max = CGAL::square(size_(p, dim, index));
+  const FT w_max = CGAL::square(query_size(p, dim, index));
+
   if(w > w_max) {
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+    std::cerr << "smart_insert_point: weight " << w
+              << " reduced to " << w_max << " (sizing field)\n";
+#endif
     w = w_max;
     add_handle_to_unchecked = true;
   }
-  
-  Vertex_handle v = insert_point(p,w,dim,index);
+  if( w < minimal_weight_) {
+    w = minimal_weight_;
+    insert_a_special_ball = true;
+  }
+  Vertex_handle v = insert_point(p,w,dim,index, insert_a_special_ball);
   if ( add_handle_to_unchecked ) { unchecked_vertices_.insert(v); }
   
   return v;
@@ -535,7 +740,7 @@ insert_balls_on_edges()
     const Curve_segment_index& curve_index = CGAL::cpp11::get<0>(*fit);
     if ( ! is_treated(curve_index) )
     {
-#ifdef PROTECTION_DEBUG
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
       std::cerr << "** treat curve #" << curve_index << std::endl;
 #endif
       const Bare_point& p = CGAL::cpp11::get<1>(*fit).first;
@@ -552,14 +757,47 @@ insert_balls_on_edges()
       }
       else
       {
-        vp = insert_curve_point(p,p_index);
+        // Even if the curve is a cycle, it can intersect other curves at
+        // its first point (here 'p'). In that case, 'p' is a corner, even
+        // if the curve is a cycle.
+        if(!c3t3_.triangulation().is_vertex(Weighted_point(p), vp))
+        {
+          // if 'p' is not a corner, find out a second point 'q' on the
+          // curve, "far" from 'p', and limit the radius of the ball of 'p'
+          // with the third of the distance from 'p' to 'q'.
+          FT p_size = query_size(p, 1, p_index);
+
+          FT curve_lenght = domain_.geodesic_distance(p, p, curve_index);
+
+          Bare_point other_point =
+            domain_.construct_point_on_curve_segment(p,
+                                                     curve_index,
+                                                     curve_lenght / 2);
+          p_size = (std::min)(p_size,
+                              compute_distance(p, other_point) / 3);
+          vp = smart_insert_point(p,
+                                  CGAL::square(p_size),
+                                  1,
+                                  p_index);
+        }
+        // No 'else' because in that case 'is_vertex(..)' already filled
+        // the variable 'vp'.
         vq = vp;
       }
       
       // Insert balls and set treated
-      insert_balls(vp, vq, curve_index);
+      // if(do_balls_intersect(vp, vq)) {
+      //   CGAL_assertion(is_special(vp) || is_special(vq));
+      // }
+      // else
+      {
+        insert_balls(vp, vq, curve_index);
+      }
       set_treated(curve_index);
     }
+    // std::stringstream s;
+    // s << "dump-mesh-curve-" << curve_index << ".binary.cgal";
+    // debug_dump_c3t3(s.str(), c3t3_);
   }
 }
 
@@ -567,29 +805,19 @@ insert_balls_on_edges()
 template <typename C3T3, typename MD, typename Sf>
 typename Protect_edges_sizing_field<C3T3, MD, Sf>::Vertex_handle
 Protect_edges_sizing_field<C3T3, MD, Sf>::
-get_vertex_corner_from_point(const Bare_point& p, const Index& p_index) const
+get_vertex_corner_from_point(const Bare_point& p, const Index&) const
 {
   // Get vertex_handle associated to corner (dim=0) point
-  FT size_p = size_(p, 0, p_index);
   Vertex_handle v;
   CGAL_assertion_code( bool q_finded = )
-  c3t3_.triangulation().is_vertex(Weighted_point(p,size_p), v);
+  c3t3_.triangulation().is_vertex(Weighted_point(p), v);
+  // Let the weight be 0, because is_vertex only locates the point, and
+  // check that the location type is VERTEX.
   CGAL_assertion( q_finded );
   return v;
 }
 
 
-template <typename C3T3, typename MD, typename Sf>
-typename Protect_edges_sizing_field<C3T3, MD, Sf>::Vertex_handle
-Protect_edges_sizing_field<C3T3, MD, Sf>::
-insert_curve_point(const Bare_point& p, const Index& p_index)
-{
-  const int p_dim = 1;
-  FT p_weight = CGAL::square(size_(p, p_dim, p_index));
-  return smart_insert_point(p, p_weight, p_dim, p_index);
-}
-  
-  
 template <typename C3T3, typename MD, typename Sf>
 void
 Protect_edges_sizing_field<C3T3, MD, Sf>::
@@ -627,6 +855,15 @@ insert_balls(const Vertex_handle& vp,
              const CGAL::Sign d_sign,
              const Curve_segment_index& curve_index)
 {
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+  std::cerr << "insert_balls(vp=" << (void*)(&*vp) << " (" << vp->point() << "),\n"
+            << "             vq=" << (void*)(&*vq) << " (" << vq->point() << "),\n"
+            << "             sp=" << sp << ",\n"
+            << "             sq=" << sq << ",\n"
+            << "              d=" << d << ",\n"
+            << "             d_sign=" << d_sign << ",\n"
+            << "             curve_index=" << curve_index << ")\n";
+#endif
   CGAL_precondition(d > 0);
   CGAL_precondition(sp <= sq);
   CGAL_precondition(sp > 0);
@@ -668,14 +905,64 @@ insert_balls(const Vertex_handle& vp,
   // =======================
   
   int n = static_cast<int>(std::floor(FT(2)*(d-sq) / (sp+sq))+.5);
+  // if( minimal_weight_ != 0 && n == 0 ) return;
+
+#ifndef CGAL_MESH_3_NO_PROTECTION_NON_LINEAR
+  // This block tries not to apply the general rule that the size of
+  // protecting balls is a linear interpolation of the size of protecting
+  // balls at corner. When the curve segment is long enough, pick a point
+  // at the middle and choose a new size.
+  if(n >= internal::max_nb_vertices_to_reevaluate_size && 
+     d >= (internal::max_nb_vertices_to_reevaluate_size * minimal_weight_)) {
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+    std::cerr << "Number of to-be-inserted balls is: " 
+              << n << "\n  between points ("
+              << vp->point() << ") and (" << vq->point()
+              << ")\n";
+#endif
+    const Bare_point new_point =
+      domain_.construct_point_on_curve_segment(vp->point().point(),
+                                               curve_index,
+                                               d_sign * d / 2);
+    const int dim = 1; // new_point is on edge
+    const Index index = domain_.index_from_curve_segment_index(curve_index);
+    const FT point_weight = CGAL::square(size_(new_point, dim, index));
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+    std::cerr << "  middle point: " << new_point << std::endl;
+    std::cerr << "  new weight: " << point_weight << std::endl;
+#endif
+    const Vertex_handle new_vertex = smart_insert_point(new_point, 
+                                                        point_weight,
+                                                        dim,
+                                                        index);
+    const FT sn = get_size(new_vertex);
+    if(sp <= sn) {
+      insert_balls(vp, new_vertex, sp, sn, d/2, d_sign, curve_index);
+    } else {
+      insert_balls(new_vertex, vp, sn, sp, d/2, -d_sign, curve_index);
+    }
+    if(sn <= sq) {
+      insert_balls(new_vertex, vq, sn, sq, d/2, d_sign, curve_index);
+    } else {
+      insert_balls(vq, new_vertex, sq, sn, d/2, -d_sign, curve_index);
+    }
+    return;
+  }
+#endif // not CGAL_MESH_3_NO_PROTECTION_NON_LINEAR
+
   FT r = (sq - sp) / FT(n+1);
-  
+
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+  std::cerr << "  n=" << n
+            << "\n  r=" << r << std::endl;
+#endif
+
+
   // Adjust size of steps, D = covered distance
   FT D = sp*FT(n+1) + FT((n+1)*(n+2)) / FT(2) * r ;
   
   FT dleft_frac = d / D;
-  CGAL_assertion(dleft_frac >= 1);
-  
+
   // Initialize step sizes
   FT step_size = sp + r;
   FT norm_step_size = dleft_frac * step_size;
@@ -686,6 +973,9 @@ insert_balls(const Vertex_handle& vp,
   Vertex_handle prev = vp;
   const Bare_point& p = vp->point().point();
 
+  // if ( (0 == n) && 
+  //      ( (d >= sp+sq) || !is_sampling_dense_enough(vp, vq) ) )
+
   // If there is some place to insert one point, insert it
   if ( (0 == n) && (d >= sp+sq) )
   {
@@ -693,6 +983,10 @@ insert_balls(const Vertex_handle& vp,
     step_size = sp + (d-sp-sq) / FT(2);
     pt_dist = d_signF * step_size;
     norm_step_size = step_size;
+  } else {
+    CGAL_assertion_code(using boost::math::float_prior);
+    CGAL_assertion(n==0 ||
+                   dleft_frac >= float_prior(float_prior(1.)));
   }
   
   // Launch balls
@@ -714,7 +1008,9 @@ insert_balls(const Vertex_handle& vp,
     Vertex_handle new_vertex = smart_insert_point(new_point, point_weight, dim, index);
     
     // Add edge to c3t3
-    c3t3_.add_to_complex(prev, new_vertex, curve_index);
+    if(!c3t3_.is_in_complex(prev, new_vertex)) {
+      c3t3_.add_to_complex(prev, new_vertex, curve_index);
+    }
     prev = new_vertex;
     
     // Step size
@@ -730,7 +1026,9 @@ insert_balls(const Vertex_handle& vp,
   // then (prev,vp) == (prev,vq)
   if ( vp != vq || n > 1 )
   {
-    c3t3_.add_to_complex(prev, vq, curve_index);
+    if(!c3t3_.is_in_complex(prev, vq)) {
+      c3t3_.add_to_complex(prev, vq, curve_index);
+    }
   }
 }
   
@@ -747,7 +1045,7 @@ refine_balls()
   int nb=0;
   while ( (!unchecked_vertices_.empty() || restart) && nb<29)
   {
-#ifdef PROTECTION_DEBUG
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
     std::cerr << "RESTART REFINE LOOP (" << nb << ")\n"
               << "\t unchecked_vertices size: " << unchecked_vertices_.size() <<"\n";
 #endif
@@ -764,6 +1062,8 @@ refine_balls()
       // If those vertices are not adjacent 
       if( non_adjacent_but_intersect(va, vb) )
       {
+        using CGAL::Mesh_3::internal::distance_divisor;
+
         // Compute correct size of balls
         const FT ab = compute_distance(va,vb);
         FT sa_new = (std::min)(ab/distance_divisor, get_size(va));
@@ -782,9 +1082,6 @@ refine_balls()
         
         if ( sb_new != get_size(vb) )
         { new_sizes[vb] = sb_new; }
-        
-        // Loop will have to be run again
-        restart = true;
       }
     }
     
@@ -793,7 +1090,19 @@ refine_balls()
          end = new_sizes.end() ; it != end ; ++it)
     {
       // Set size of the ball to new value
-      change_ball_size(it->first,it->second);
+      if(minimal_size_ != FT() && it->second < minimal_size_) {
+        if(!is_special(it->first)) {
+          change_ball_size(it->first,minimal_size_,true); // special ball
+                  
+          // Loop will have to be run again
+          restart = true;
+        }
+      } else {
+        change_ball_size(it->first,it->second);
+                
+        // Loop will have to be run again
+        restart = true;
+      }
     }
     
     // Check edges
@@ -809,33 +1118,52 @@ non_adjacent_but_intersect(const Vertex_handle& va, const Vertex_handle& vb) con
 {
   if ( ! c3t3_.is_in_complex(va,vb) )
   {
-    typename Gt::Construct_sphere_3 sphere = 
-      c3t3_.triangulation().geom_traits().construct_sphere_3_object();
-    
-    typename Gt::Do_intersect_3 do_intersect = 
-      c3t3_.triangulation().geom_traits().do_intersect_3_object();
-    
-    const Bare_point& a = va->point().point();
-    const Bare_point& b = vb->point().point();
-    
-    const FT& sq_ra = va->point().weight();
-    const FT& sq_rb = vb->point().weight();
-    
-    return do_intersect(sphere(a, sq_ra), sphere(b, sq_rb));
+    return do_balls_intersect(va, vb);
   }
   
   return false;
 }
 
 
+template <typename C3T3, typename MD, typename Sf>
+bool
+Protect_edges_sizing_field<C3T3, MD, Sf>::
+do_balls_intersect(const Vertex_handle& va, const Vertex_handle& vb) const
+{
+  typename Gt::Construct_sphere_3 sphere = 
+    c3t3_.triangulation().geom_traits().construct_sphere_3_object();
+    
+  typename Gt::Do_intersect_3 do_intersect = 
+    c3t3_.triangulation().geom_traits().do_intersect_3_object();
+    
+  const Bare_point& a = va->point().point();
+  const Bare_point& b = vb->point().point();
+    
+  const FT& sq_ra = va->point().weight();
+  const FT& sq_rb = vb->point().weight();
+    
+  return do_intersect(sphere(a, sq_ra), sphere(b, sq_rb));
+}
+
 template <typename C3T3, typename MD, typename Sf>
 typename Protect_edges_sizing_field<C3T3, MD, Sf>::Vertex_handle
 Protect_edges_sizing_field<C3T3, MD, Sf>::
-change_ball_size(const Vertex_handle& v, const FT size)
+change_ball_size(const Vertex_handle& v, const FT size, const bool special_ball)
 {
   // Check if there is something to do
-  if ( get_size(v) == size )
-  { return v; }
+  Weight w = CGAL::square(size);
+  // if(v->point().weight() == w)
+  // { return v; }
+
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+  std::cerr << "change_ball_size(v=" << (void*)(&*v) 
+            << " (" << v->point()
+            << ") dim=" << c3t3_.in_dimension(v) 
+            << " index=" << c3t3_.index(v)
+            << " ,\n"
+            << "                 size=" << size 
+            << ", special_ball=" << std::boolalpha << special_ball << std::endl;
+#endif
   
   // Get incident vertices along c3t3 edge
   Incident_vertices incident_vertices;
@@ -849,10 +1177,11 @@ change_ball_size(const Vertex_handle& v, const FT size)
   }
   
   
+  
   // Store point data
-  Index index = v->index();
-  int dim = v->in_dimension();
-  Bare_point p = v->point().point();
+  Index index = c3t3_.index(v);
+  int dim = get_dimension(v);
+  Bare_point p = v->point().point();  
 
   // Remove v from corners
   boost::optional<Corner_index> corner_index;
@@ -861,9 +1190,21 @@ change_ball_size(const Vertex_handle& v, const FT size)
     corner_index = c3t3_.corner_index(v);
     c3t3_.remove_from_complex(v);
   }
+
   // Change v size
   c3t3_.triangulation().remove(v);
-  Vertex_handle new_v = insert_point(p, size*size, dim, index);
+ 
+  CGAL_assertion_code(Tr& tr = c3t3_.triangulation());
+  CGAL_assertion_code(Cell_handle ch = tr.locate(p));
+  CGAL_assertion_code(std::vector<Vertex_handle> hidden_vertices);
+  CGAL_assertion_code(tr.vertices_inside_conflict_zone(Weighted_point(p, w),
+                                                       ch,
+                                                       std::back_inserter(hidden_vertices)));
+
+  Vertex_handle new_v = insert_point(p, w , dim, index, special_ball);
+  CGAL_assertion(hidden_vertices.empty());
+
+  CGAL_assertion( (! special_ball) || is_special(new_v) ); 
   
   // TODO: ensure that this condition is always satisfied (Pedro's code ?)
   CGAL_assertion(v==new_v);
@@ -872,7 +1213,13 @@ change_ball_size(const Vertex_handle& v, const FT size)
   // Restore v in corners
   if ( corner_index )
   {
+    // As C3t3::add_to_complex modifies the 'in_dimension' of the vertex,
+    // we need to backup and re-set the 'is_special' marker after.
+    const bool special_ball = is_special(new_v);
     c3t3_.add_to_complex(new_v,*corner_index);
+    if(special_ball) {
+        set_special(new_v);
+    }
   }
   
   // Restore c3t3 edges
@@ -895,6 +1242,9 @@ void
 Protect_edges_sizing_field<C3T3, MD, Sf>::
 check_and_repopulate_edges()
 {
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+  std::cerr << "check_and_repopulate_edges()\n";
+#endif
   std::set<Vertex_handle> vertices;
   std::copy( unchecked_vertices_.begin(), unchecked_vertices_.end(),
              std::inserter(vertices,vertices.begin()) );
@@ -913,12 +1263,9 @@ check_and_repopulate_edges()
     for ( typename Vertex_vector::iterator vit = erased_vertices.begin(), 
          vend = erased_vertices.end() ; vit != vend ; ++vit )
     {
-      if ( c3t3_.in_dimension(*vit) > 0 )
-      {
-        vertices.erase(*vit);
-      }
+      vertices.erase(*vit);
     }
-  }  
+  }
 }
 
   
@@ -928,6 +1275,14 @@ OutputIterator
 Protect_edges_sizing_field<C3T3, MD, Sf>::
 check_and_fix_vertex_along_edge(const Vertex_handle& v, OutputIterator out)
 {
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+  std::cerr << "check_and_fix_vertex_along_edge(" 
+            << (void*)(&*v) << "= (" << v->point() 
+            << ") dim=" << get_dimension(v)
+            << " index=" << c3t3_.index(v)
+            << " special=" << std::boolalpha << is_special(v)
+            << ")\n";
+#endif
   // If v is a corner, then all incident edges have to be checked
   if ( c3t3_.is_in_complex(v) )
   {
@@ -937,8 +1292,17 @@ check_and_fix_vertex_along_edge(const Vertex_handle& v, OutputIterator out)
   // Get incident vertices along c3t3 edge
   Incident_vertices incident_vertices;
   c3t3_.adjacent_vertices_in_complex(v, std::back_inserter(incident_vertices));
-  CGAL_assertion(incident_vertices.size() == 2);
-  
+  CGAL_assertion(v->is_special()
+                 || incident_vertices.size() == 0
+                 || incident_vertices.size() == 1
+                 || incident_vertices.size() == 2);
+  if(incident_vertices.size() == 0) return out;
+  // The size of 'incident_vertices' can be 0 if v is a ball that covers
+  // entirely a closed curve. 
+  // The size can also be 1 if the curve is a cycle, and the temporary
+  // mesh is only two balls on the cycle: then each ball has only one
+  // neighbor.
+
   // Walk along edge to find the edge piece which is not correctly sampled
   typedef std::list<Vertex_handle> Vertex_list;
   Vertex_list to_repopulate;
@@ -949,13 +1313,19 @@ check_and_fix_vertex_along_edge(const Vertex_handle& v, OutputIterator out)
 
   // Walk following direction (v,previous)
   walk_along_edge(v, previous, true, std::front_inserter(to_repopulate));
-  
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+  std::cerr <<  "to_repopulate.size()=" << to_repopulate.size() << "\n";
+#endif // CGAL_MESH_3_PROTECTION_DEBUG  
+
   // Check whether a complete circle has been discovered or not
   if (   to_repopulate.size() == 1
       || to_repopulate.front() != to_repopulate.back() )
   {
     // Walk in other direction (v,next)
     walk_along_edge(v, next, true, std::back_inserter(to_repopulate));    
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+    std::cerr <<  "to_repopulate.size()=" << to_repopulate.size() << "\n";
+#endif // CGAL_MESH_3_PROTECTION_DEBUG  
   }
   
   // If only v is in to_repopulate, there is nothing to do
@@ -984,6 +1354,7 @@ bool
 Protect_edges_sizing_field<C3T3, MD, Sf>::
 is_sampling_dense_enough(const Vertex_handle& v1, const Vertex_handle& v2) const
 {
+  using CGAL::Mesh_3::internal::min_intersection_factor;
   CGAL_precondition(c3t3_.is_in_complex(v1,v2));
 
   // Get sizes
@@ -1065,6 +1436,15 @@ void
 Protect_edges_sizing_field<C3T3, MD, Sf>::
 repopulate(InputIterator begin, InputIterator last, const Curve_segment_index& index)
 {
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+  std::cerr << "repopulate(begin=" << (void*)(&**begin) 
+            << " (" << (*begin)->point() << "),\n"
+            << "            last=" << (void*)(&**last)
+            << " (" << (*last)->point() << ")\n"
+            << "                  distance(begin, last)=" 
+            << std::distance(begin, last) << ",\n"
+            << "           index=" << index << ")\n";
+#endif
   CGAL_assertion( std::distance(begin,last) >= 0 );
   
   // May happen
@@ -1088,6 +1468,23 @@ repopulate(InputIterator begin, InputIterator last, const Curve_segment_index& i
   current = begin;
   while ( ++current != last )
   {
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+    std::cerr << "Removal of ";
+    if(is_special(*current)) std::cerr << "SPECIAL ";
+    std::cerr << "protecting ball "
+              << (*current)->point();
+    switch(get_dimension(*current)) {
+    case 0:
+      std::cerr << " on corner #";
+      break;
+    case 1:
+      std::cerr << " on curve #";
+      break;
+    default:
+      std::cerr << " ERROR dim=" << get_dimension(*current)  << " index=";
+    }
+    std::cerr  << c3t3_.index(*current) << std::endl;
+#endif // CGAL_MESH_3_PROTECTION_DEBUG
     c3t3_.triangulation().remove(*current);
   }
   
@@ -1113,6 +1510,15 @@ void
 Protect_edges_sizing_field<C3T3, MD, Sf>::
 analyze_and_repopulate(InputIterator begin, InputIterator last, const Curve_segment_index& index)
 {
+#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+  std::cerr << "analyze_and_repopulate(begin=" << (void*)(&**begin)
+            << " (" << (*begin)->point() << "),\n"
+            << "                       last=" << (void*)(&**last)
+            << " (" << (*last)->point() << ")\n"
+            << "                              distance(begin, last)=" 
+            << std::distance(begin, last) << ",\n"
+            << "                       index=" << index << ")\n";
+#endif
   CGAL_assertion( std::distance(begin,last) >= 0 );
   
   // May happen
