@@ -16,7 +16,7 @@
 // $Id$
 //
 //
-// Author(s)     : Stéphane Tayeb
+// Author(s)     : Laurent Rineau, Stéphane Tayeb
 //
 //******************************************************************************
 // File Description :
@@ -139,6 +139,15 @@ public:
   /// Gets the point to insert from the element to refine
   Point refinement_point_impl(const Facet& facet) const
   {
+#ifdef CGAL_MESHES_DEBUG_REFINEMENT_POINTS
+    const Cell_handle c = facet.first;
+    const int i = facet.second;
+    std::cerr << "Facet (" 
+              << c->vertex((i+1)&3)->point() << " , "
+              << c->vertex((i+2)&3)->point() << " , "
+              << c->vertex((i+3)&3)->point() << ") : refinement point is "
+              << get_facet_surface_center(facet) << std::endl;
+#endif
     CGAL_assertion (is_facet_on_surface(facet));
     last_vertex_index_ = get_facet_surface_center_index(facet);
     return get_facet_surface_center(facet);
@@ -196,6 +205,29 @@ public:
     return "#facets to refine";
   }
   
+  /// for debugging
+  std::string display_dual(Facet f) const 
+  {
+    std::stringstream stream;
+    stream.precision(17);
+    Object dual = r_tr_.dual(f);
+
+    if ( const Segment_3* p_segment = object_cast<Segment_3>(&dual) ) {
+      stream << "Segment(" << p_segment->source()
+             << " , " << p_segment->target() << ")";
+    }
+    else if ( const Ray_3* p_ray = object_cast<Ray_3>(&dual) ) {
+      stream << "Ray(" << p_ray->point(0)
+             << " , " << p_ray->point(1) 
+             << "), with vector (" << p_ray->to_vector() << ")";
+    }
+    else if ( const Line_3* p_line = object_cast<Line_3>(&dual) ) {
+      stream << "Line(point=" << p_line->point(0)
+             << " , vector=" << p_line->to_vector() << ")";
+    }
+    return stream.str();
+  }
+
 #ifdef CGAL_MESH_3_MESHER_STATUS_ACTIVATED
   std::size_t queue_size() const { return this->size(); }
 #endif
@@ -314,7 +346,7 @@ private:
    * @param facet The input facet
    * @return \c true if \c facet is on surface, \c false otherwise
    */
-  Facet_properties compute_facet_properties(const Facet& facet) const;
+  void compute_facet_properties(const Facet& facet, Facet_properties& fp ) const;
 
   /// Returns true if point encroaches facet
   bool is_facet_encroached(const Facet& facet, const Point& point) const;
@@ -554,23 +586,25 @@ before_insertion_impl(const Facet& facet,
   // called from a Mesh_3 visitor.
   if ( !source_facet_is_in_conflict && facet != Facet()  )
   {
-    const Facet source_other_side = mirror_facet(facet);
+    using boost::io::group;
+    using std::setprecision;
+
     std::stringstream error_msg;
     error_msg <<
       boost::format("Mesh_3 ERROR: "
                     "A facet is not in conflict with its refinement point!\n"
                     "Debugging informations:\n"
                     "  Facet: (%1%, %2%) = (%6%, %7%, %8%)\n"
-                    "  Dual: (%3%, %4%)\n"
+                    "  Dual: %3%\n"
                     "  Refinement point: %5%\n")
-      % (&*facet.first)
-      % facet.second
-      % triangulation_ref_impl().dual(facet.first)
-      % triangulation_ref_impl().dual(source_other_side.first)
-      % point
-      % facet.first->vertex((facet.second + 1)&3)->point()
-      % facet.first->vertex((facet.second + 2)&3)->point()
-      % facet.first->vertex((facet.second + 3)&3)->point();
+      % group(setprecision(17), (&*facet.first))
+      % group(setprecision(17), facet.second)
+      % display_dual(facet)
+      % 0 // dummy: %4% no longer used
+      % group(setprecision(17), point)
+      % group(setprecision(17), facet.first->vertex((facet.second + 1)&3)->point())
+      % group(setprecision(17), facet.first->vertex((facet.second + 2)&3)->point())
+      % group(setprecision(17), facet.first->vertex((facet.second + 3)&3)->point());
 
     CGAL_error_msg(error_msg.str().c_str());
   }
@@ -647,7 +681,8 @@ Refine_facets_3<Tr,Cr,MD,C3T3_,P_,C_>::
 treat_new_facet(Facet& facet)
 {
   // Treat facet
-  Facet_properties properties = compute_facet_properties(facet);
+  Facet_properties properties;
+  compute_facet_properties(facet, properties);
   if ( properties )
   {
     const Surface_patch_index& surface_index = CGAL::cpp11::get<0>(*properties);
@@ -679,9 +714,10 @@ treat_new_facet(Facet& facet)
 
 
 template<class Tr, class Cr, class MD, class C3T3_, class P_, class C_>
-typename Refine_facets_3<Tr,Cr,MD,C3T3_,P_,C_>::Facet_properties
+void
 Refine_facets_3<Tr,Cr,MD,C3T3_,P_,C_>::
-compute_facet_properties(const Facet& facet) const
+compute_facet_properties(const Facet& facet,
+                         Facet_properties& fp) const
 {
   //-------------------------------------------------------
   // Facet must be finite
@@ -705,11 +741,13 @@ compute_facet_properties(const Facet& facet) const
   // If the dual is a segment
   if ( const Segment_3* p_segment = object_cast<Segment_3>(&dual) )
   {
-    if (is_degenerate(*p_segment)) { return Facet_properties(); }
+    if (is_degenerate(*p_segment)) { fp = Facet_properties(); return; }
 
     // If facet is on surface, compute intersection point and return true
+#ifndef CGAL_MESH_3_NO_LONGER_CALLS_DO_INTERSECT_3
     Surface_patch surface = do_intersect_surface(*p_segment);
     if ( surface )
+#endif // not CGAL_MESH_3_NO_LONGER_CALLS_DO_INTERSECT_3
     {
       typename MD::Construct_intersection construct_intersection =
           r_oracle_.construct_intersection_object();
@@ -727,9 +765,19 @@ compute_facet_properties(const Facet& facet) const
       }
 
       Intersection intersect = construct_intersection(segment);
-      return Facet_properties(CGAL::cpp11::make_tuple(*surface,
-                                                CGAL::cpp11::get<1>(intersect),
-                                                CGAL::cpp11::get<0>(intersect)));
+#ifdef CGAL_MESH_3_NO_LONGER_CALLS_DO_INTERSECT_3
+      // In the following, CGAL::cpp0x::get<2>(intersect) == 0 is a way to
+      // test "intersect == Intersection()" (aka empty intersection), but
+      // the later does not work.
+      Surface_patch surface = 
+        (CGAL::cpp0x::get<2>(intersect) == 0) ? Surface_patch() : 
+        r_oracle_.surface_patch_index(CGAL::cpp0x::get<1>(intersect));
+      if(surface)
+#endif // CGAL_MESH_3_NO_LONGER_CALLS_DO_INTERSECT_3
+      fp =  Facet_properties(CGAL::cpp0x::make_tuple(*surface,
+                                                     CGAL::cpp0x::get<1>(intersect),
+                                                     CGAL::cpp0x::get<0>(intersect)));
+      return;
     }
   }
   // If the dual is a ray
@@ -740,19 +788,31 @@ compute_facet_properties(const Facet& facet) const
     // a ray constructed with a point with very big coordinates, and a
     // vector with small coordinates. Its can happen than the
     // constructed ray is degenerate (the point(1) of the ray is
-    // point(0) plus a vector whose coordinates are espilon).
-    if (is_degenerate(*p_ray)) { return Facet_properties(); }
+    // point(0) plus a vector whose coordinates are epsilon).
+    if (is_degenerate(*p_ray)) { fp = Facet_properties(); return; }
 
+#ifndef CGAL_MESH_3_NO_LONGER_CALLS_DO_INTERSECT_3
     Surface_patch surface = do_intersect_surface(*p_ray);
     if ( surface )
+#endif // not CGAL_MESH_3_NO_LONGER_CALLS_DO_INTERSECT_3
     {
       typename MD::Construct_intersection construct_intersection =
           r_oracle_.construct_intersection_object();
 
       Intersection intersect = construct_intersection(*p_ray);
-      return Facet_properties(CGAL::cpp11::make_tuple(*surface,
-                                                CGAL::cpp11::get<1>(intersect),
-                                                CGAL::cpp11::get<0>(intersect)));
+#ifdef CGAL_MESH_3_NO_LONGER_CALLS_DO_INTERSECT_3
+      Surface_patch surface = 
+        (CGAL::cpp0x::get<2>(intersect) == 0) ? Surface_patch() : 
+        r_oracle_.surface_patch_index(CGAL::cpp0x::get<1>(intersect));
+      if(surface)
+#endif // CGAL_MESH_3_NO_LONGER_CALLS_DO_INTERSECT_3
+        {
+          fp = Facet_properties(CGAL::cpp0x::make_tuple(*surface,
+                                                        CGAL::cpp0x::get<1>(intersect),
+                                                        CGAL::cpp0x::get<0>(intersect)));
+          return;
+        }
+      
     }
   }
   // If the dual is a line
@@ -778,9 +838,10 @@ compute_facet_properties(const Facet& facet) const
       }
 
       Intersection intersect = construct_intersection(line);
-      return Facet_properties(CGAL::cpp11::make_tuple(*surface,
-                                                CGAL::cpp11::get<1>(intersect),
-                                                CGAL::cpp11::get<0>(intersect)));
+      fp = Facet_properties(CGAL::cpp11::make_tuple(*surface,
+                                                    CGAL::cpp11::get<1>(intersect),
+                                                    CGAL::cpp11::get<0>(intersect)));
+      return;
     }
   }
   else
@@ -792,7 +853,7 @@ compute_facet_properties(const Facet& facet) const
     CGAL_error();
   }
 
-  return Facet_properties();
+  fp = Facet_properties();
 }
 
 
