@@ -23,6 +23,8 @@
 #ifndef CGAL_MESH_3_SLIVERS_EXUDER_H
 #define CGAL_MESH_3_SLIVERS_EXUDER_H
 
+#include <CGAL/Mesh_3/config.h>
+
 #include <CGAL/Double_map.h>
 #include <CGAL/iterator.h>
 #include <map>
@@ -32,6 +34,7 @@
 #include <iomanip> // std::setprecision
 #include <iostream> // std::cerr/cout
 #include <boost/iterator/transform_iterator.hpp>
+#include <boost/function_output_iterator.hpp>
 #include <boost/bind.hpp>
 #include <boost/format.hpp>
 
@@ -81,19 +84,22 @@ namespace Mesh_3 {
     // returns the distance d(v1, v2).
     // It is used in Sliver_exuder, to constructor a transform iterator.
     template <typename Gt, typename Vertex_handle>
-    class Distance_from_v :
-    public std::unary_function<Vertex_handle, typename Gt::RT>
+    class Min_distance_from_v :
+    public std::unary_function<Vertex_handle, void>
     {
       const Vertex_handle * v;
       Gt gt;
+      double & dist;
+
     public:
-      Distance_from_v(const Vertex_handle& vh,
-                      Gt geom_traits = Gt())
-      : v(&vh), gt(geom_traits)
+      Min_distance_from_v(const Vertex_handle& vh,
+                          double& dist,
+                          Gt geom_traits = Gt())
+        : v(&vh), gt(geom_traits), dist(dist)
       {
       }
       
-      typename Gt::RT
+      void
       operator()(const Vertex_handle& vh) const
       {
         typedef typename Gt::Compute_squared_distance_3
@@ -101,9 +107,12 @@ namespace Mesh_3 {
         Compute_squared_distance_3 distance =
         gt.compute_squared_distance_3_object();
         
-        return distance((*v)->point(), vh->point());
+        const double d = CGAL::to_double(distance((*v)->point(), vh->point()));
+        if(d < dist){
+          dist = d;
+        }
       }
-    }; // end class Distance_from_v
+    }; // end class Min_distance_from_v
     
   } // end namespace details
   
@@ -177,7 +186,7 @@ public: // methods
    * max_weight(v) < d*dist(v,nearest_vertice(v))
    */
   Slivers_exuder(C3T3& c3t3,
-                 const SliverCriteria& criteria = SliverCriteria(),
+                 const SliverCriteria& criterion,
                  double d = 0.45);
   
   /**
@@ -186,10 +195,9 @@ public: // methods
    * quality below this bound will be pumped
    */
   Mesh_optimization_return_code
-  operator()(double criterion_value_limit = SliverCriteria::default_value,
-             Visitor visitor = Visitor())
+  operator()(Visitor visitor = Visitor())
   {
-    return pump_vertices<true>(criterion_value_limit, visitor);
+    return pump_vertices<true>(sliver_criteria_.sliver_bound(), visitor);
   }
   
   /// Time accessors
@@ -278,12 +286,12 @@ private:
   /**
    * Initialization
    */
-  void init(double radius_ratio_limit = SliverCriteria::default_value )
+  void init(double limit_value)
   {
-    if ( 0 < radius_ratio_limit )
-      sliver_bound_ = radius_ratio_limit;
+    if ( 0 < limit_value )
+      sliver_criteria_.set_sliver_bound(limit_value);
     else
-      sliver_bound_ = SliverCriteria::max_value;
+      sliver_criteria_.set_sliver_bound(SliverCriteria::max_value);
       
     cells_queue_.clear();
     initialize_cells_priority_queue();
@@ -299,9 +307,10 @@ private:
         cit != c3t3_.cells_in_complex_end() ;
         ++cit)
     {
-      const double value = sliver_criteria_(tr_.tetrahedron(cit));
-      
-      if( value < sliver_bound_ )
+      const double value 
+        = sliver_criteria_(cit);
+
+      if( value < sliver_criteria_.sliver_bound() )
         cells_queue_.insert(cit, value);
     }
   }
@@ -330,20 +339,14 @@ private:
    */
   double get_closest_vertice_squared_distance(const Vertex_handle& vh) const
   {
-    using boost::make_transform_iterator;
-    
-    std::vector<Vertex_handle> adjacent_vertices;
-    adjacent_vertices.reserve(128);
-    tr_.adjacent_vertices(vh, std::back_inserter(adjacent_vertices));
-    
-    details::Distance_from_v<Geom_traits, Vertex_handle>
-      distance_from_v(vh, Geom_traits());
-    
-    return *(std::min_element(make_transform_iterator(adjacent_vertices.begin(),
-                                                      distance_from_v),
-                              make_transform_iterator(adjacent_vertices.end(),
-                                                      distance_from_v)));
+
+    double dist = (std::numeric_limits<double>::max)();
+    details::Min_distance_from_v<Geom_traits, Vertex_handle>
+      min_distance_from_v(vh, dist);
+
+    tr_.adjacent_vertices(vh, boost::make_function_output_iterator(min_distance_from_v));
     
+    return dist;
   }
   
   /** 
@@ -449,9 +452,10 @@ private:
         cit != c3t3_.cells_in_complex_end() ;
         ++cit)
     {
-      const double value = sliver_criteria_(tr_.tetrahedron(cit));
-      
-      if( value < sliver_bound_ )
+      const double value = 
+        sliver_criteria_(cit);
+
+      if( value < sliver_criteria_.sliver_bound() )
         return false;
     }
     
@@ -465,7 +469,6 @@ private:
   C3T3& c3t3_;
   Tr& tr_;
   double sq_delta_;
-  double sliver_bound_;
   
   int num_of_pumped_vertices_;
   int num_of_ignored_vertices_;
@@ -544,7 +547,6 @@ Slivers_exuder(C3T3& c3t3, const SC& criteria, double d)
   : c3t3_(c3t3)
   , tr_(c3t3_.triangulation())
   , sq_delta_(d*d)
-  , sliver_bound_(0)
   , num_of_pumped_vertices_(0)
   , num_of_ignored_vertices_(0)
   , num_of_treated_vertices_(0)
@@ -622,15 +624,26 @@ pump_vertices(double sliver_criterion_limit,
 #ifdef CGAL_MESH_3_EXUDER_VERBOSE   
   std::cerr << std::endl;
   std::cerr << "Total exuding time: " << running_time_.time() << "s";
-  std::cerr << std::endl << std::endl;
+  std::cerr << std::endl;
 #endif // CGAL_MESH_3_EXUDER_VERBOSE  
   
-  if ( is_time_limit_reached() )
+  if ( is_time_limit_reached() ) {
+#ifdef CGAL_MESH_3_EXUDER_VERBOSE
+    std::cerr << "Exuding return code: TIME_LIMIT_REACHED\n\n";
+#endif // CGAL_MESH_3_EXUDER_VERBOSE
     return TIME_LIMIT_REACHED;
-
-  if ( check_sliver_bound() )
+  }
+  
+  if ( check_sliver_bound() ) {
+#ifdef CGAL_MESH_3_EXUDER_VERBOSE
+    std::cerr << "Exuding return code: BOUND_REACHED\n\n";
+#endif // CGAL_MESH_3_EXUDER_VERBOSE
     return BOUND_REACHED;
-    
+  }
+  
+#ifdef CGAL_MESH_3_EXUDER_VERBOSE
+    std::cerr << "Exuding return code: CANT_IMPROVE_ANYMORE\n\n";
+#endif // CGAL_MESH_3_EXUDER_VERBOSE
   return CANT_IMPROVE_ANYMORE;
   
 } // end function pump_vertices
@@ -681,8 +694,7 @@ initialize_prestar_and_criterion_values(const Vertex_handle& v,
     // Sliver criterion values initialization
     if( c3t3_.is_in_complex(*cit) )
     {
-      double r = sliver_criteria_(tr_.tetrahedron(*cit));
-      criterion_values[f] = r;
+      criterion_values[f] = sliver_criteria_(*cit);
     }
     
 
@@ -881,10 +893,11 @@ get_umbrella(const Facet_vector& facets,
   typename Facet_vector::const_iterator fit = facets.begin();
   for ( ; fit != facets.end() ; ++fit )
   {
-    Ordered_edge edge = get_opposite_ordered_edge(*fit, v);
-    
     if ( c3t3_.is_in_complex(*fit) )
+    {
+      Ordered_edge edge = get_opposite_ordered_edge(*fit, v);
       umbrella.insert(std::make_pair(edge, c3t3_.surface_patch_index(*fit)));
+    }
   }
   
   return umbrella;
@@ -910,6 +923,7 @@ restore_cells_and_boundary_facets(
       cit != new_cells.end();
       ++cit)
   {
+    (*cit)->invalidate_circumcenter();
     const int index = (*cit)->index(new_vertex);
     const Facet new_facet = std::make_pair(*cit, index);
     const Facet new_facet_from_outside = tr_.mirror_facet(new_facet);
@@ -933,9 +947,10 @@ restore_cells_and_boundary_facets(
     // the maximum, push it in the cells queue.
     if( c3t3_.is_in_complex(*cit) )
     {
-      double criterion_value = sliver_criteria_(tr_.tetrahedron(*cit));
-
-      if( criterion_value < sliver_bound_ )
+      double criterion_value 
+        = sliver_criteria_(*cit);
+      
+      if( criterion_value < sliver_criteria_.sliver_bound() )
         cells_queue_.insert(*cit, criterion_value);
     }
   }  
@@ -1007,6 +1022,8 @@ Slivers_exuder<C3T3,SC,V_,FT>::
 update_mesh(const Weighted_point& new_point,
             const Vertex_handle& old_vertex)
 {
+  CGAL_assertion_code(std::size_t nb_vert = 
+                      tr_.number_of_vertices());
   Cell_vector deleted_cells;
   Facet_vector internal_facets;
   Facet_vector boundary_facets;
@@ -1042,10 +1059,13 @@ update_mesh(const Weighted_point& new_point,
   Vertex_handle new_vertex = tr_.insert(new_point);
   c3t3_.set_dimension(new_vertex,dimension);
   c3t3_.set_index(new_vertex,vertice_index);
-  
+
+  CGAL_assertion(nb_vert == tr_.number_of_vertices());
+    
   // Restore mesh
   restore_cells_and_boundary_facets(boundary_facets_from_outside, new_vertex);
   restore_internal_facets(umbrella, new_vertex);
+  CGAL_assertion(nb_vert == tr_.number_of_vertices());
 }
 
   
