@@ -26,6 +26,7 @@
 #define CGAL_MESH_3_TRIANGULATION_HELPERS_H
 
 #include <vector>
+#include <CGAL/squared_distance_3.h>
 
 namespace CGAL {
 
@@ -69,9 +70,19 @@ public:
    * Returns true if moving \c v to \c p makes no topological
    * change in \c tr
    */
+  bool no_topological_change(const Tr& tr,
+                             const Vertex_handle& v,
+                             const Point_3& p,
+                             Cell_vector& cells_tos) const;
+
+
   bool no_topological_change(const Tr& tr,
                              const Vertex_handle& v,
                              const Point_3& p) const;
+
+  bool inside_protecting_balls(const Tr& tr,
+                               const Vertex_handle& v,
+                               const Point_3& p) const;
   
 private:
   /**
@@ -103,14 +114,17 @@ bool
 Triangulation_helpers<Tr>::
 no_topological_change(const Tr& tr,
                       const Vertex_handle& v0,
-                      const Point_3& p) const
+                      const Point_3& p,
+                      Cell_vector& cells_tos) const
 {
   bool np = true;
   Point_3 fp = v0->point();
   v0->set_point(p);
   
-  Cell_vector cells_tos;
-  tr.incident_cells(v0, std::back_inserter(cells_tos));
+/// @TODO: One can do the same checks without the set_point.
+/// In the following orientation or side_of_power_sphere tests, just
+/// replace v0->point() by p.
+
   if(!well_oriented(tr, cells_tos)) 
   {
     // Reset (restore) v0
@@ -166,7 +180,38 @@ no_topological_change(const Tr& tr,
   return np;
 }
   
+
+template<typename Tr>
+bool
+Triangulation_helpers<Tr>::
+no_topological_change(const Tr& tr,
+                      const Vertex_handle& v0,
+                      const Point_3& p) const
+{
+  Cell_vector cells_tos;
+  cells_tos.reserve(64);
+  tr.incident_cells(v0, std::back_inserter(cells_tos));
+  return no_topological_change(tr, v0, p, cells_tos);
+}
+
+template<typename Tr>
+bool
+Triangulation_helpers<Tr>::
+inside_protecting_balls(const Tr& tr,
+                        const Vertex_handle& v,
+                        const Point_3& p) const
+{
+  Vertex_handle nv = tr.nearest_power_vertex(p, v->cell());
+  if(nv->point().weight() > 0)
+    return CGAL::compare_squared_distance(p, nv->point(),
+                         nv->point().weight()) != CGAL::LARGER;
+  return false;
+}
+
   
+/// This function well_oriented is called by no_topological_change after a
+/// v->set_point(p)
+/// @TODO: One can do the same checks without the set_point.
 template<typename Tr>
 bool
 Triangulation_helpers<Tr>::
