@@ -25,7 +25,9 @@
 #ifndef CGAL_MESH_3_VERTEX_PERTURBATION_H
 #define CGAL_MESH_3_VERTEX_PERTURBATION_H
 
+#include <CGAL/Mesh_3/config.h>
 
+#include <CGAL/Timer.h>
 #include <CGAL/Mesh_3/C3T3_helpers.h>
 #include <CGAL/Mesh_3/Triangulation_helpers.h>
 
@@ -420,7 +422,7 @@ protected:
     C3T3_helpers helper(c3t3, domain);
     
     modified_vertices.clear();
-    
+
     // norm depends on the local size of the mesh
     FT sq_norm = this->compute_perturbation_sq_amplitude(v, c3t3, sq_step_size_);
     FT step_length = CGAL::sqrt(sq_norm/sq_length(gradient_vector));
@@ -444,14 +446,15 @@ protected:
     }
     
     // Topology could not change moving this vertex
-    if ( i > max_step_nb_ )
+    if ( i > max_step_nb_ 
+      || Th().inside_protecting_balls(c3t3.triangulation(), v, final_loc))
       return std::make_pair(false,v);
     
     // we know that there will be a combinatorial change
-    return helper.update_mesh(final_loc,
-                              v,
-                              criterion,
-                              std::back_inserter(modified_vertices));
+    return helper.update_mesh_topo_change(final_loc,
+                                          v,
+                                          criterion,
+                                          std::back_inserter(modified_vertices));
   }
   
   
@@ -986,7 +989,7 @@ public:
     : max_try_nb_(max_try_nb) 
     , sphere_radius_(sphere_radius)
     , sphere_sq_radius_(sphere_radius*sphere_radius)
-    , generator_(boost::uint32_t(std::time(0)))
+    , generator_() // initialize the random generator deterministically
     , uni_dist_(0,1)
     , random_(generator_, uni_dist_) {}
   
@@ -1162,6 +1165,8 @@ private:
                      const FT& sliver_bound,
                      std::vector<Vertex_handle>& modified_vertices) const
   {
+    typedef Triangulation_helpers<typename C3T3::Triangulation> Th;
+
     modified_vertices.clear();
 
     // Create an helper
@@ -1173,10 +1178,10 @@ private:
     const Point_3 initial_location = v->point();
     
     // Initialize loop variables
-    bool min_angle_increased = false;
+    bool criterion_improved = false;
     Vertex_handle moving_vertex = v;
     Point_3 best_location = initial_location;
-    std::vector<Vertex_handle> best_vertices;
+    std::set<Vertex_handle> mod_vertices;
     
     // TODO: use no_topo_change to compute new_angle without moving everything
     unsigned int try_nb = 0;
@@ -1190,6 +1195,11 @@ private:
       if ( c3t3.in_dimension(moving_vertex) < 3 )
         new_location = helper.project_on_surface(new_location, moving_vertex);
       
+      // check that we don't insert a vertex inside a protecting ball
+      if(Th().inside_protecting_balls(c3t3.triangulation(), 
+                                      moving_vertex, new_location))
+        continue;
+
       // try to move vertex
       std::vector<Vertex_handle> tmp_mod_vertices;
       std::pair<bool,Vertex_handle> update =
@@ -1197,51 +1207,48 @@ private:
                            moving_vertex,
                            criterion,
                            std::back_inserter(tmp_mod_vertices));
-      
+
       // get new vertex
       moving_vertex = update.second;
       
       if ( update.first )
       {
-        min_angle_increased = true;
+        criterion_improved = true;
         best_location = moving_vertex->point();
-        //best_vertices = tmp_mod_vertices;
-        
-        for ( typename std::vector<Vertex_handle>::iterator it = tmp_mod_vertices.begin() ;
-             it != tmp_mod_vertices.end() ;
-             ++it )
-        {
-          if ( std::find(best_vertices.begin(), best_vertices.end(), *it)
-              == best_vertices.end() )
-          { 
-            best_vertices.push_back(*it);
-          }
-        }
-        
+
+        mod_vertices.insert(tmp_mod_vertices.begin(), tmp_mod_vertices.end());
+
+        std::size_t nois;
+#ifdef CGAL_NEW_INCIDENT_SLIVERS
+        nois  = helper.number_of_incident_slivers(moving_vertex, criterion, sliver_bound);
+#else
         std::vector<Cell_handle> new_slivers =
           helper.incident_slivers(moving_vertex, criterion, sliver_bound);
-
+        nois = new_slivers.size();
+#endif
+        
         // If sliver number has decreased, we won
-        if ( new_slivers.size() < slivers.size() )
+        if(nois < slivers.size() )
         {
-          std::copy(best_vertices.begin(),
-                    best_vertices.end(),
+          std::copy(mod_vertices.begin(),
+                    mod_vertices.end(),
                     std::back_inserter(modified_vertices));
           
           return std::make_pair(true,moving_vertex);
         }
       }
-    }
+    }//end while ( ++try_nb <= Base::max_try_nb() )
+
     
-    if ( min_angle_increased )
+    if ( criterion_improved )
     {
-      std::copy(best_vertices.begin(),
-                best_vertices.end(),
+      std::copy(mod_vertices.begin(),
+                mod_vertices.end(),
                 std::back_inserter(modified_vertices));
     }
     
     // Moving vertex is located on the best location
-    return std::make_pair(min_angle_increased,moving_vertex);
+    return std::make_pair(criterion_improved, moving_vertex);
   }
   
 private:
