@@ -1,4 +1,5 @@
 // Copyright (c) 2006-2009 INRIA Sophia-Antipolis (France).
+// Copyright (c) 2011      GeometryFactory Sarl (France)
 // All rights reserved.
 //
 // This file is part of CGAL (www.cgal.org).
@@ -22,6 +23,8 @@
 #ifndef CGAL_MESH_TRIANGULATION_3_H
 #define CGAL_MESH_TRIANGULATION_3_H
 
+#include <CGAL/Mesh_3/config.h>
+
 #include <CGAL/Kernel_traits.h>
 
 #include <CGAL/Regular_triangulation_3.h>
@@ -29,46 +32,54 @@
 #include <CGAL/Mesh_3/Robust_weighted_circumcenter_filtered_traits_3.h>
 
 #include <CGAL/Mesh_vertex_base_3.h>
-#include <CGAL/Mesh_cell_base_3.h>
+#include <CGAL/Compact_mesh_cell_base_3.h>
 
 namespace CGAL {
-  
+
   namespace details {
-    
+
     template<typename K>
     struct Mesh_geom_traits_generator
     {
     private:
       typedef Robust_weighted_circumcenter_filtered_traits_3<K>
         Geom_traits;
-      
+
     public:
       typedef Geom_traits type;
       typedef type Type;
     };  // end struct Mesh_geom_traits_generator
-    
+
   }  // end namespace details
-  
-  
+
+
 // Struct Mesh_triangulation_3
 //
-template<class MD, class K=typename Kernel_traits<MD>::Kernel>
+template<class MD,
+         class K=typename Kernel_traits<MD>::Kernel,
+         class Concurrency_tag = Default,
+         class Vertex_base_ = Default,
+         class Cell_base_   = Default>
 struct Mesh_triangulation_3
   {
 private:
   typedef typename details::Mesh_geom_traits_generator<K>::type Geom_traits;
-  typedef Mesh_vertex_base_3<Geom_traits, MD>                   Vertex_base;
-  typedef Mesh_cell_base_3<Geom_traits, MD>                     Cell_base;
+
+  typedef typename Default::Get<
+    Vertex_base_,
+    Mesh_vertex_base_3<Geom_traits, MD> >::type                 Vertex_base;
+  typedef typename Default::Get<
+    Cell_base_,
+    Compact_mesh_cell_base_3<Geom_traits, MD> >::type           Cell_base;
+
   typedef Triangulation_data_structure_3<Vertex_base,Cell_base> Tds;
   typedef Regular_triangulation_3<Geom_traits, Tds>             Triangulation;
-  
+
 public:
   typedef Triangulation type;
   typedef type Type;
 };  // end struct Mesh_triangulation_3
 
-
-
 }  // end namespace CGAL
 
 #endif // CGAL_MESH_TRIANGULATION_3_H
