@@ -12,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/releases/CGAL-4.0-branch/Min_annulus_d/include/CGAL/Min_annulus_d.h $
-// $Id: Min_annulus_d.h 67117 2012-01-13 18:14:48Z lrineau $
+// $URL$
+// $Id$
 // 
 //
 // Author(s)     : Sven Schoenherr <sven@inf.ethz.ch>
@@ -311,7 +311,7 @@ public:
   // access to point set
   int  ambient_dimension( ) const { return d; }
     
-  int  number_of_points( ) const { return points.size(); }
+  int  number_of_points( ) const { return static_cast<int>(points.size()); }
     
   Point_iterator  points_begin( ) const { return points.begin(); }
   Point_iterator  points_end  ( ) const { return points.end  (); }
@@ -345,8 +345,8 @@ public:
 						   std::bind2nd( std::divides<int>(), 2)));
   }
     
-  int  number_of_inner_support_points() const { return inner_indices.size();}
-  int  number_of_outer_support_points() const { return outer_indices.size();}
+  int  number_of_inner_support_points() const { return static_cast<int>(inner_indices.size());}
+  int  number_of_outer_support_points() const { return static_cast<int>(outer_indices.size());}
     
   Inner_support_point_iterator
   inner_support_points_begin() const
@@ -486,7 +486,7 @@ public:
   template < class InputIterator >
   void
   insert( InputIterator first, InputIterator last)
-  { CGAL_optimisation_precondition_code( int old_n = points.size());
+  { CGAL_optimisation_precondition_code( std::size_t old_n = points.size());
   points.insert( points.end(), first, last);
   set_dimension();
   CGAL_optimisation_precondition_msg( check_dimension( old_n),
@@ -587,7 +587,7 @@ private:
     
   // check dimension of input points
   bool
-  check_dimension( unsigned int  offset = 0)
+  check_dimension( std::size_t  offset = 0)
   { return ( std::find_if( points.begin()+offset, points.end(),
 			   CGAL::compose1_1( std::bind2nd(
 							  std::not_equal_to<int>(), d),
@@ -637,7 +637,7 @@ private:
       c_vector[ 2*j+1] = -sum;
     }
         
-    LP lp (2*points.size(), d+2, 
+    LP lp (2*static_cast<int>(points.size()), d+2, 
 	   A_iterator ( boost::counting_iterator<int>(0), 
 			A_matrix (d, da_coord, points.begin())),
 	   B_iterator ( boost::counting_iterator<int>(0), 
