@@ -23,54 +23,51 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413 
 // (ACS -- Algorithms for Complex Shapes)
 
-#ifndef CGAL_ALGEBRAIC_KERNEL_FOR_CIRCLES_POLYNOMIALS_2_2_H
-#define CGAL_ALGEBRAIC_KERNEL_FOR_CIRCLES_POLYNOMIALS_2_2_H
-
-//////////// FIXME - pb RT (cas general Polynomial_2_2) ou FT (ici)
+#ifndef CGAL_ALGEBRAIC_KERNEL_FOR_CIRCLES_POLYNOMIALS_1_2_H
+#define CGAL_ALGEBRAIC_KERNEL_FOR_CIRCLES_POLYNOMIALS_1_2_H
 
 #include <CGAL/enum.h>
 
 namespace CGAL {
 
-// polynomials of the form (X-a)^2 + (Y-b)^2 - R^2
-template < typename FT_ >
-class Polynomial_for_circles_2_2
+template < typename RT_ >
+class Polynomial_1_2
 {
-  FT_ rep[3]; // stores a, b, R^2
+  RT_ rep[3]; // stores a, b, c for line ax+by+c=0
   
 public:
   
-  typedef FT_ FT;
+  typedef RT_ RT;
   
-  Polynomial_for_circles_2_2(){}
+  Polynomial_1_2(){}
   
-  Polynomial_for_circles_2_2(const FT & a, const FT & b, const FT & rsq)
+  Polynomial_1_2(const RT & a, const RT & b, const RT & c)
   { 
     rep[0]=a;
     rep[1]=b;
-    rep[2]=rsq;
+    rep[2]=c;
   }
 
-  const FT & a() const
+  const RT & a() const
   { return rep[0]; }
 
-  const FT & b() const
+  const RT & b() const
   { return rep[1]; }
-  
-  const FT & r_sq() const
+
+  const RT & c() const
   { return rep[2]; }
 };
 
-template < typename FT >
+template < typename RT >
 bool 
-operator == ( const Polynomial_for_circles_2_2<FT> & p1,
-	      const Polynomial_for_circles_2_2<FT> & p2 )
+operator == ( const Polynomial_1_2<RT> & p1,
+	      const Polynomial_1_2<RT> & p2 )
 {
   return( (p1.a() == p2.a()) && 
               (p1.b() == p2.b()) &&
-              (p1.r_sq() == p2.r_sq()) );
+              (p1.c() == p2.c()) );
 }
-
+    
 } //namespace CGAL
 
-#endif //CGAL_ALGEBRAIC_KERNEL_FOR_CIRCLES_POLYNOMIALS_2_2_H
+#endif //CGAL_ALGEBRAIC_KERNEL_FOR_CIRCLES_POLYNOMIALS_1_2_H
