@@ -23,6 +23,7 @@
 #include <CGAL/boost/graph/named_function_params.h>
 
 #include <CGAL/Surface_mesh_simplification/Detail/Edge_collapse.h>
+#include <CGAL/Surface_mesh_simplification/Detail/Common.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk.h>
 
 namespace CGAL {
@@ -35,23 +36,25 @@ template<class ECM
         ,class VertexIndexMap
         ,class EdgeIndexMap
         ,class EdgeIsBorderMap
+        ,class EdgeIsConstrainedMap
         ,class GetCost
         ,class GetPlacement
         ,class Visitor
         >
-int edge_collapse ( ECM&                    aSurface
-                  , ShouldStop       const& aShould_stop
+int edge_collapse ( ECM&                       aSurface
+                  , ShouldStop          const& aShould_stop
                   
                   // optional mesh information policies 
-                  , VertexIndexMap   const& aVertex_index_map     // defaults to get(vertex_index,aSurface) 
-                  , EdgeIndexMap     const& aEdge_index_map       // defaults to get(edge_index,aSurface) 
-                  , EdgeIsBorderMap  const& aEdge_is_border_map   // defaults to get(edge_is_border,aSurface) 
+                  , VertexIndexMap       const& aVertex_index_map     // defaults to get(vertex_index,aSurface)
+                  , EdgeIndexMap         const& aEdge_index_map       // defaults to get(edge_index,aSurface)
+                  , EdgeIsBorderMap      const& aEdge_is_border_map   // defaults to get(edge_is_border,aSurface)
+                  , EdgeIsConstrainedMap const& aEdge_is_constrained_map   // defaults to No_constrained_edge_map<ECM>()
                   
                   // optional strategy policies - defaults to LindstomTurk
-                  , GetCost          const& aGet_cost 
-                  , GetPlacement     const& aGet_placement
+                  , GetCost              const& aGet_cost
+                  , GetPlacement         const& aGet_placement
                   
-                  , Visitor                 aVisitor 
+                  , Visitor                     aVisitor
                   ) 
 {
   typedef EdgeCollapse< ECM
@@ -59,6 +62,7 @@ int edge_collapse ( ECM&                    aSurface
                       , VertexIndexMap
                       , EdgeIndexMap
                       , EdgeIsBorderMap
+                      , EdgeIsConstrainedMap
                       , GetCost
                       , GetPlacement
                       , Visitor
@@ -70,6 +74,7 @@ int edge_collapse ( ECM&                    aSurface
                      , aVertex_index_map
                      , aEdge_index_map
                      , aEdge_is_border_map
+                     , aEdge_is_constrained_map
                      , aGet_cost
                      , aGet_placement
                      , aVisitor
@@ -110,10 +115,11 @@ int edge_collapse ( ECM& aSurface
                       ,choose_const_pmap(get_param(aParams,boost::vertex_index),aSurface,boost::vertex_index)
                       ,choose_const_pmap(get_param(aParams,boost::edge_index),aSurface,boost::edge_index)
                       ,choose_const_pmap(get_param(aParams,edge_is_border),aSurface,edge_is_border)
+                      ,choose_param     (get_param(aParams,edge_is_constrained),No_constrained_edge_map<ECM>())
                       ,choose_param     (get_param(aParams,get_cost_policy), LindstromTurk_cost<ECM>())
                       ,choose_param     (get_param(aParams,get_placement_policy), LindstromTurk_placement<ECM>())
                       ,choose_param     (get_param(aParams,vis), Dummy_visitor())
-                      ) ;
+                      );
 
 }
 
