@@ -38,6 +38,7 @@ class Triangulation_cell_base_with_circumcenter_3
 
   mutable Point_3 * circumcenter_;
 
+public:
   void invalidate_circumcenter()
   {
       if (circumcenter_) {
@@ -122,7 +123,14 @@ public:
 	                                           this->vertex(1)->point(),
 						   this->vertex(2)->point(),
 						   this->vertex(3)->point()));
+      } else {
+        CGAL_expensive_assertion(gt.construct_circumcenter_3_object()
+                                 (this->vertex(0)->point(),
+                                  this->vertex(1)->point(),
+                                  this->vertex(2)->point(),
+                                  this->vertex(3)->point()) == *circumcenter);
       }
+
       return *circumcenter_;
   }
 };
