@@ -126,6 +126,17 @@ public:
     ++*this;
     return tmp;
   }
+
+  Self& operator--() {
+    --nt;
+    return *this;
+  }
+
+  Self  operator--(int) {
+    Self tmp = *this;
+    --*this;
+    return tmp;
+  }
 };
 
 template < class HDS >
@@ -197,6 +208,17 @@ public:
     ++*this;
     return tmp;
   }
+
+  Self& operator--() {
+    --nt;
+    return *this;
+  }
+
+  Self  operator--(int) {
+    Self tmp = *this;
+    --*this;
+    return tmp;
+  }
 };
 
 template < class HDS >
