@@ -68,6 +68,8 @@ namespace CGAL {
   enum get_cost_policy_params_t      { get_cost_policy_params      } ;
   enum get_placement_policy_t        { get_placement_policy        } ;
   enum get_placement_policy_params_t { get_placement_policy_params } ;
+  enum edge_is_constrained_t         { edge_is_constrained        } ;
+  enum edge_is_constrained_params_t  { edge_is_constrained_params } ;
   
 #if BOOST_VERSION >= 105100
   template <typename T, typename Tag, typename Base = boost::no_property>
@@ -166,6 +168,22 @@ namespace CGAL {
       typedef cgal_bgl_named_params<GetPlacementParams, get_placement_policy_params_t, self> Params;
       return Params(p, *this);
     }
+
+    template <typename EdgeIsConstrained>
+    cgal_bgl_named_params<EdgeIsConstrained, edge_is_constrained_t, self>
+    edge_is_constrained_map(const EdgeIsConstrained& em) const
+    {
+      typedef cgal_bgl_named_params<EdgeIsConstrained, edge_is_constrained_t, self> Params;
+      return Params(em, *this);
+    }
+
+    template <typename EdgeIsConstrainedParams>
+    cgal_bgl_named_params<EdgeIsConstrainedParams, edge_is_constrained_params_t, self>
+    edge_is_constrained_map_params(const EdgeIsConstrainedParams& em) const
+    {
+      typedef cgal_bgl_named_params<EdgeIsConstrainedParams, edge_is_constrained_params_t, self> Params;
+      return Params(em, *this);
+    }
   };
 #else
   template <typename T, typename Tag, typename Base = boost::no_property>
@@ -270,6 +288,22 @@ namespace CGAL {
       typedef cgal_bgl_named_params<GetPlacementParams, get_placement_policy_params_t, self> Params;
       return Params(p, *this);
     }
+
+    template <typename EdgeIsConstrained>
+    cgal_bgl_named_params<EdgeIsConstrained, edge_is_constrained_t, self>
+    edge_is_constrained_map(const EdgeIsConstrained& em) const
+    {
+      typedef cgal_bgl_named_params<EdgeIsConstrained, edge_is_constrained_t, self> Params;
+      return Params(em, *this);
+    }
+
+    template <typename EdgeIsConstrainedParams>
+    cgal_bgl_named_params<EdgeIsConstrainedParams, edge_is_constrained_params_t, self>
+    edge_is_constrained_map_params(const EdgeIsConstrainedParams& em) const
+    {
+      typedef cgal_bgl_named_params<EdgeIsConstrainedParams, edge_is_constrained_params_t, self> Params;
+      return Params(em, *this);
+    }
   };
 
   template <class Tag1, class Tag2, class T1, class Base>
@@ -373,6 +407,22 @@ namespace CGAL {
     typedef cgal_bgl_named_params<GetPlacementParams, get_placement_policy_params_t> Params;
     return Params(p);
   }
+
+  template <typename EdgeIsConstrained>
+  cgal_bgl_named_params<EdgeIsConstrained, edge_is_constrained_t>
+  edge_is_constrained_map(const EdgeIsConstrained& em)
+  {
+    typedef cgal_bgl_named_params<EdgeIsConstrained, edge_is_constrained_t> Params;
+    return Params(em);
+  }
+
+  template <typename EdgeIsConstrainedParams>
+  cgal_bgl_named_params<EdgeIsConstrainedParams, edge_is_constrained_params_t>
+  edge_is_constrained_map_params(const EdgeIsConstrainedParams& em)
+  {
+    typedef cgal_bgl_named_params<EdgeIsConstrainedParams, edge_is_constrained_params_t> Params;
+    return Params(em);
+  }
 } //namespace CGAL
 
 #if BOOST_VERSION >= 105100
