@@ -35,6 +35,8 @@
 #ifdef CGAL_USE_GMP
 #  include <CGAL/Gmpz.h>
 #  include <CGAL/Gmpq.h>
+#  include <CGAL/Gmpzf.h>
+#  include <CGAL/Mpzf.h>
 #endif
 #ifdef CGAL_USE_GMPXX
 #  include <CGAL/gmpxx.h>
@@ -53,76 +55,124 @@ class Expr;
 
 namespace CGAL { namespace internal {
 
-// A class which tells the prefered "exact number type" corresponding to a type.
+// Two classes which tell the prefered "exact number types" corresponding to a type.
 
 // The default template chooses Gmpq or Quotient<MP_Float>.
 // It should support the built-in types.
 template < typename >
-struct Exact_type_selector
+struct Exact_field_selector
 #ifdef CGAL_USE_GMP
 { typedef Gmpq Type; };
 #else
 { typedef Quotient<MP_Float> Type; };
 #endif
 
+// By default, a field is a safe choice of ring.
+template < typename T >
+struct Exact_ring_selector : Exact_field_selector < T > { };
+
 template <>
-struct Exact_type_selector<MP_Float>
+struct Exact_ring_selector<double>
+#ifdef CGAL_HAS_MPZF
+{ typedef Mpzf Type; };
+#elif defined(CGAL_HAS_THREADS) || !defined(CGAL_USE_GMP)
+{ typedef MP_Float Type; };
+#else
+{ typedef Gmpzf Type; };
+#endif
+
+template <>
+struct Exact_ring_selector<float> : Exact_ring_selector<double> { };
+
+template <>
+struct Exact_field_selector<MP_Float>
 { typedef Quotient<MP_Float> Type; };
 
 template <>
-struct Exact_type_selector<Quotient<MP_Float> >
+struct Exact_ring_selector<MP_Float>
+{ typedef MP_Float Type; };
+
+template <>
+struct Exact_field_selector<Quotient<MP_Float> >
 { typedef Quotient<MP_Float> Type; };
 
 // And we specialize for the following types :
 #ifdef CGAL_USE_GMP
 template <>
-struct Exact_type_selector<Gmpz>
+struct Exact_field_selector<Gmpz>
 { typedef Gmpq  Type; };
 
 template <>
-struct Exact_type_selector<Gmpq>
+struct Exact_ring_selector<Gmpz>
+{ typedef Gmpz  Type; };
+
+template <>
+struct Exact_ring_selector<Gmpzf>
+{ typedef Gmpzf Type; };
+
+template <>
+struct Exact_field_selector<Gmpq>
 { typedef Gmpq  Type; };
 #endif
 
 #ifdef CGAL_USE_GMPXX
 template <>
-struct Exact_type_selector< ::mpz_class>
+struct Exact_field_selector< ::mpz_class>
 { typedef ::mpq_class  Type; };
 
 template <>
-struct Exact_type_selector< ::mpq_class>
+struct Exact_ring_selector< ::mpz_class>
+{ typedef ::mpz_class  Type; };
+
+template <>
+struct Exact_field_selector< ::mpq_class>
 { typedef ::mpq_class  Type; };
 #endif
 
 #ifdef CGAL_USE_LEDA
 template <>
-struct Exact_type_selector<leda_integer>
+struct Exact_field_selector<leda_integer>
 { typedef leda_rational  Type; };
 
 template <>
-struct Exact_type_selector<leda_rational>
+struct Exact_ring_selector<leda_integer>
+{ typedef leda_integer   Type; };
+
+template <>
+struct Exact_field_selector<leda_rational>
 { typedef leda_rational  Type; };
 
 template <>
-struct Exact_type_selector<leda_real>
+struct Exact_field_selector<leda_real>
 { typedef leda_real  Type; };
 #endif
 
 #ifdef CGAL_USE_CORE
 template <>
-struct Exact_type_selector<CORE::Expr>
+struct Exact_field_selector<CORE::Expr>
 { typedef CORE::Expr  Type; };
 #endif
 
 template < typename ET >
-struct Exact_type_selector<Lazy_exact_nt<ET> >
+struct Exact_field_selector<Lazy_exact_nt<ET> >
+: Exact_field_selector<ET>
 {
   // We have a choice here :
   // - using ET gets rid of the DAG computation as well as redoing the interval
   // - using Lazy_exact_nt<ET> might use sharper intervals.
-  typedef ET  Type;
+  // typedef ET  Type;
   // typedef Lazy_exact_nt<ET>  Type;
 };
+template < typename ET >
+struct Exact_ring_selector<Lazy_exact_nt<ET> >
+: Exact_ring_selector<ET>
+{};
+
+#ifndef CGAL_NO_DEPRECATED_CODE
+// Added for backward compatibility
+template < typename ET >
+struct Exact_type_selector : Exact_field_selector< ET > {};
+#endif
 
 } } // namespace CGAL::internal
 
