@@ -383,20 +383,19 @@ namespace internal {
 
 } // namespace internal
 
-template <class K>
-bool do_intersect(const CGAL::Triangle_3<K>& triangle,
-		  const CGAL::Bbox_3& bbox)
-{
-  return typename K::Do_intersect_3()(triangle, bbox);
+template<typename K>
+bool do_intersect(const CGAL::Bbox_3 a,
+                  const Triangle_3<K>& b) {
+  return K().do_intersect_3_object()(a, b);
 }
 
-template <class K>
-bool do_intersect(const CGAL::Bbox_3& bbox,
-		  const CGAL::Triangle_3<K>& triangle)
-{
-  return typename K::Do_intersect_3()(triangle, bbox);
+template<typename K>
+bool do_intersect(const Triangle_3<K>& a,
+                  const CGAL::Bbox_3& b) {
+  return K().do_intersect_3_object()(a, b);
 }
 
+
 } //namespace CGAL
 
 #endif  // CGAL_INTERNAL_INTERSECTIONS_3_BBOX_3_TRIANGLE_3_DO_INTERSECT_H
